#------------------------------------------------------------------------------
# Programm         : G r a p h 2 D . p m
#------------------------------------------------------------------------------
# Autor            : Christian Dhl
# Erstellt         : 08. Februar 2004
# Zuletzt gendert : 17. Januar 2005
# Aufgabe          : Graphenklasse mit Koordinaten fr zweidimensionale
#                    Darstelung.
#------------------------------------------------------------------------------

#
# Falls man irgendwann dreidimensionale Koordinaten will, sollte man die
# Koordinaten als eigene Klasse auslagern und diese Klasse parametrisieren.
# (und an die Abstnde denken ....)
#

#
# TODO:
#
# Setzen auf Radius am Setter/Getter vorbei ist sehr unschn, aber wie
# soll man die Bindung an Tk anders machen?

#------------------------------------------------------------------------------
# Globale private Datei-Variablen:
#------------------------------------------------------------------------------
my $Debug  = 0;


#------------------------------------------------------------------------------
# Klasse           : G r a p h 2 d
# abgeleitet von   : Graph
#------------------------------------------------------------------------------
# Autor            : Christian Dhl
# Aufgabe          : Eine Graphen-Klasse zur zweidimensionalen Darstellung.
# Attribute        : radius                - Gre der gezeichneten cken
#                                            (Radius == 3 -> Kreis in 7x7 Pixeln)
#                    delta                 - Abstand, den Ecken voneinander haben mssen,
#                                            innerhalb dieses Abstandes werden auch
#                                            Klicks der entsprechenden Ecke zugeordnet.
#                                            Wird mit der Funktion r2d() berechnet.
#                    eckenfarbe            - Farbe der Ecken (default: rot)
#                    kantenfarbe           - Farbe der Kanten (default: schwarz)
#                    hintergrund           - Farbe des Hintergrundes (default: wei)
#                    gitterzeigen          - Hilfs-Gitter anzeigen?
#                    gitterweite           - Weite des Gitters
#                    rastimgitter          - Im Gitter einrasten?
#                    gitterfarbe           - Farbe des Gitters
#                    hintergrundbild       - Hintergrundbild
#                    kantendicke           - Defaultwert fr Kantendicke
#                    dateiname             - Dateiname
#                    _ecken                - anonymes Array von Ecke-Objekten
# Klassen-Methoden : new                   - Erstellt ein Objekt dieser Klasse.
# Objekt-Methoden  : clone                 - Erzeugt ein geclontes Objekt (eine
#                                            Kopie, die referenzierte Member-
#                                            hashwerte richtig kopiert und
#                                            nicht nur eine Kopie der Referenz
#                                            anlegt.
#                    radius                - Getter / Setter
#                    delta                 - Getter / Setter
#                    eckenfarbe            - Getter / Setter
#                    kantenfarbe           - Getter / Setter
#                    hintergrund           - Getter / Setter
#                    gitterzeigen          - Getter / Setter
#                    gitterweite           - Getter / Setter
#                    rastimgitter          - Getter / Setter
#                    gitterfarbe           - Getter / Setter
#                    hintergrundbild       - Getter / Setter
#                    kantendicke           - Getter / Setter
#                    dateiname             - Getter / Setter
#                    _ecken                - Interner Getter / Setter
#                    setze_ecken           - Ersetzt die Eckenmenge
#                    loesche_ecken         - Lscht die Eckenmenge
#                    ecke                  - Getter/Setter ber Index fr
#                                            eine Ecke aus der Menge
#                    entnehme_ecke         - spliced eine Ecke heraus
#                    fuege_ecke_hinzu      - pushed eine Ecke hinzu
#                    x_von_a               - Gibt die x-Koordinate der Anfangs-
#                                            ecke der bergebenen Kante aus.
#                    y_von_a               - Gibt die y-Koordinate der Anfangs-
#                                            ecke der bergebenen Kante aus.
#                    x_von_e               - Gibt die x-Koordinate der End-
#                                            secke der bergebenen Kante aus.
#                    y_von_e               - Gibt die y-Koordinate der End-
#                                            ecke der bergebenen Kante aus.
#                    x_von_a_index         - Gibt die x-Koordinate der Anfangs-
#                                            ecke der Kante aus, die zu dem
#                                            bergebenen Index gehrt.
#                    y_von_a_index         - Gibt die y-Koordinate der Anfangs-
#                                            ecke der Kante aus, die zu dem
#                                            bergebenen Index gehrt.
#                    x_von_e_index         - Gibt die x-Koordinate der End-
#                                            ecke der Kante aus, die zu dem
#                                            bergebenen Index gehrt.
#                    y_von_e_index         - Gibt die y-Koordinate der End-
#                                            ecke der Kante aus, die zu dem
#                                            bergebenen Index gehrt.
#                    ecke_einfuegen        - Ecke einfgen
#                    ecke_loeschen         - Ecke lschen
#                    ecke_mergen           - Ecke mergen (Ecke wird gelscht
#                                            und die Nachbarn der Ecke alle
#                                            miteinander durch Kanten ver-
#                                            bunden)
#                    clear                 - Lscht alles.
#                    ecke_zu_nah           - Testet, ob ein Koordinatenpaar zu
#                                            nah an einer der vorhandenen
#                                            Ecke ist (anhand der Koordinaten
#                                            und delta).
#                    ecke_zu_nah_ohne      - Wie ecke_zu_nah unter Ausschlu
#                                            einer angegebenen Ecke (zum
#                                            Test beim Verschieben von Ecken).
#                    meint_ecke            - Testet, ob ein Koordinatenpaar
#                                            eindeutig eine Ecke meint.
#                    meint_kante           - Testet, ob ein Koordinatenpaar
#                                            eindeutig eine Kante meint.
#                    r2d                   - Berechnet delta aus dem Radius.
#                    struktur_einfuegen    - Fgt eine angegebene Struktur wie
#                                            etwa den K_5 in den Graphen ein.
#                    erzeuge_liniengraphen - Erzeugt aus dem Graphen seinen
#                                            Liniengraphen.
#                    toLaTeX               - Export ins LaTeX-Format.
#                    nach_links            - Bewegt den Graphen nach links.
#                    nach_rechts           - Bewegt den Graphen nach rechts.
#                    nach_oben             - Bewegt den Graphen nach oben.
#                    nach_unten            - Bewegt den Graphen nach unten.
#                    zentrieren            - Zentriert Graphen.
#                    zentrieren_skalieren  - Zentriert und skaliert Graphen.
#                    kleiner               - Verkleinert den Graphen.
#                    groesser              - vergroessert den Graphen.
#                    save_ged              - Speichert den Graphen im ged-
#                                            Format.
#                    load_ged              - Ldt den Graphen im ged-Format.
#                    save_ged2             - Speichert den Graphen im ged2-
#                                            Format.
#                    load_ged2             - Ldt den Graphen im ged2-Format.
#                    save_ged3             - Speichert den Graphen im ged3-
#                                            Format.
#                    load_ged3             - Ldt den Graphen im ged3-Format.
#------------------------------------------------------------------------------
# 0.0.1 - 08.02.2004 - CD - erstellt
# 0.0.2 - 03.01.2005 - CD - Neue Attribute gitterzeigen, gitterweite,
#                           rastimgitter und gitterfarbe.
# 0.0.3 - 14.01.2005 - CD - Neues Attribut hintergrundbild
# 0.0.4 - 16.01.2005 - CD - Neues Attribut kantendicke
#                           Neues Attribut dateiname
#                           Neues Speicherformat GED3 (XML)
# 0.0.5 - 17.01.2005 - CD - Verschieben und zentrieren und skalieren.
#------------------------------------------------------------------------------
package Graph::GED::Graph2D;

use strict;
use warnings;

use Graph::GED::Ecke;

use base qw/Graph::GED::Graph/;

#------------------------------------------------------------------------------
# Globale ffentliche Klassen-Variablen:
#------------------------------------------------------------------------------


our $MinX  = -1;    # Diese Variablen geben die minimalen und
our $MaxX  = -1;    # maximalen Koordinaten der Zeichenflche an.
our $MinY  = -1;    # Diese Werte mssen vom Programm gesetzt
our $MaxY  = -1;    # werden, das Graph2D verwendet.


sub new {
    #--------------------------------------------------------------------------
    # Methode      : n e w
    # Klasse       : Graph::GED::Graph2D
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Beschreibung : Erzeugt ein neues Graph2D-Objekt.
    # Parameter    : Klasse
    # Rckgabewert : Das geblesste neue Objekt.
    #--------------------------------------------------------------------------
    # 0.0.1 - 08.02.2004 - CD - erstellt
    # 0.0.2 - 03.01.2005 - CD - Neue Attribute gitterzeigen, gitterweite,
    #                           rastimgitter und gitterfarbe initialisiert.
    # 0.0.3 - 14.01.2005 - CD - Neues Attribut hintergrundbild
    # 0.0.4 - 16.01.2005 - CD - Neues Attribut kantendicke
    #                           Neues Attribut dateiname
    #--------------------------------------------------------------------------

    my $self  = shift;
    my $class = (ref $self) || $self;

    #--------------------------------------------------------------------------
    # Ausgabe der Aufrufparameter:
    #--------------------------------------------------------------------------
    print "Graph::GED::Graph2D::new\n",
          "\tself       = '$self'\n",
          "\tclass      = '$class'\n",
          "\tParameter  = @_\n",
          if $Debug > 1;


    $self = $class->SUPER::new();


    #--------------------------------------------------------------------------
    # Nun setzen wir noch ein paar Attribute:
    #--------------------------------------------------------------------------
    $self->radius         (        3);
    $self->delta          (       -1);
    $self->eckenfarbe     ('#FF0000');
    $self->kantenfarbe    ('#000000');
    $self->hintergrund    ('#FFFFFF');
    $self->gitterzeigen   (        0);
    $self->gitterweite    (       25);
    $self->rastimgitter   (        0);
    $self->gitterfarbe    ('#C0C0C0');
    $self->_ecken         (       []);
    $self->hintergrundbild(       '');
    $self->kantendicke    (        1);
    $self->dateiname      (       '');


    #--------------------------------------------------------------------------
    # ... und geben das erzeugte Objekt zurck:
    #--------------------------------------------------------------------------
    return $self;

} # sub Graph::GED::Graph2D::new


sub clone {
    #--------------------------------------------------------------------------
    # Methode      : c l o n e
    # Klasse       : Graph::GED::Graph2D
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Erzeugt ein geclontes Objekt (eine Kopie, die referenzier-
    #                te Memberhashwerte richtig kopiert und nicht nur eine
    #                Kopie der Referenz anlegt.
    # Parameter    : Ev. ein von einer abgeleiteten Klasse nach unten durchge-
    #                reichtes Objekt, das wir wiederum durchreichen sollen und
    #                anschlieend seine in dieser Klasse hinzugefgten Kon-
    #                tainer kopieren sollen.
    # Rckgabewert : Das geclonte Objekt.
    #--------------------------------------------------------------------------
    # 0.0.1 - 10.02.2004 - CD - erstellt
    # 0.0.2 - 11.02.2004 - CD - Funktionalitt fertig gestellt
    #--------------------------------------------------------------------------

    my $self = shift;
    my $ursp = shift;
    my $clone;

    print "Graph::GED::Graph2D::Clone\n" if $Debug;

    #--------------------------------------------------------------------------
    # Bekommen wir von "oben" ein Objekt?
    #--------------------------------------------------------------------------
    if (defined $ursp) {
        $clone = $ursp;
    }
    #--------------------------------------------------------------------------
    # Falls nicht erzeugen wir das Objekt und kopieren unsere Daten flach:
    #--------------------------------------------------------------------------
    else {
        $clone = $self->new();
        %$clone = %$self;
    }

    #--------------------------------------------------------------------------
    # Geerbte Teile clonen:
    #--------------------------------------------------------------------------
    $self->SUPER::clone($clone);

    #--------------------------------------------------------------------------
    # Dann Kontainer kopieren:
    #--------------------------------------------------------------------------
    # (Hier mssen auch alle Elemente des Arrays gecloned werden, da das ja
    # auch Referenzen sind!)
    #--------------------------------------------------------------------------
    my @ecken;
    for (@{$self->_ecken()}) {
        my $ecke = $_->clone();
        print "Originalecke: ", $_   , "\n",
              "cloneecke   : ", $ecke, "\n"
              if $Debug > 1;
        push @ecken, $ecke;
    }
    $clone->_ecken(\@ecken);        # Referenz auf Kopie ablegen


    return $clone;
} # sub Graph::GED::Graph2D::clone


#------------------------------------------------------------------------------
# The getter/setter methods:
#------------------------------------------------------------------------------
sub radius           { return shift->_property('radius',           @_) }
sub delta            { return shift->_property('delta',            @_) }
sub eckenfarbe       { return shift->_property('eckenfarbe',       @_) }
sub kantenfarbe      { return shift->_property('kantenfarbe',      @_) }
sub hintergrund      { return shift->_property('hintergrund',      @_) }
sub gitterzeigen     { return shift->_property('gitterzeigen',     @_) }
sub gitterweite      { return shift->_property('gitterweite',      @_) }
sub rastimgitter     { return shift->_property('rastimgitter',     @_) }
sub gitterfarbe      { return shift->_property('gitterfarbe',      @_) }
sub hintergrundbild  { return shift->_property('hintergrundbild',  @_) }
sub kantendicke      { return shift->_property('kantendicke',      @_) }
sub dateiname        { return shift->_property('dateiname',        @_) }
sub _ecken           { return shift->_property('_ecken',           @_) }



#==============================================================================
#
#
#   H I L F S M E T H O D E N
#
#   (nur diese verwenden _ecken)
#
#==============================================================================

sub setze_ecken {
    #--------------------------------------------------------------------------
    # Methode      : s e t z e _ e c k e n
    # Klasse       : Graph
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Ersetzt die gesamte Eckenmenge.
    #                Die alte Menge wird vorher durch das Setzen auf undef
    #                freigegeben.
    # Parameter    : Referenz auf Array (mit Ecke-Objekten)
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 10.02.2004 - CD - erstellt
    #--------------------------------------------------------------------------

    $_[0]->_ecken(undef);
    $_[0]->_ecken($_[1]);

} # sub Graph::GED::Graph2D::setze_ecken


sub loesche_ecken {
    #--------------------------------------------------------------------------
    # Methode      : l o e s c h e _ e c k e n
    # Klasse       : Graph
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Setzt die gesamte Eckenmenge auf eine leere Menge.
    #                Die alte Menge wird vorher durch das Setzen auf undef
    #                freigegeben.
    # Parameter    : keine
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 10.02.2004 - CD - erstellt
    #--------------------------------------------------------------------------

    $_[0]->_ecken(undef);
    $_[0]->_ecken([]);

} # sub Graph::GED::Graph2D::loesche_ecken


sub ecke {
    #--------------------------------------------------------------------------
    # Methode      : e c k e
    # Klasse       : Graph
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Setter / Getter fr ein Ecke(nobjekt).
    # Parameter    : 1) Index
    #                2) Wert (optional)
    # Rckgabewert : Der aktuelle Wert (im Falle des Getters) oder der alte
    #                Wert (im falle des Setters).
    #--------------------------------------------------------------------------
    # 0.0.1 - 10.02.2004 - CD - erstellt
    #--------------------------------------------------------------------------

    if (@_ == 3) {
        my $aw = $_[0]->_ecken()->[$_[1]];
        $_[0]->_ecken()->[$_[1]] = $_[2];
        return $aw;
    }
    else {
        return $_[0]->_ecken()->[$_[1]];
    }

} # sub Graph::GED::Graph2D::ecke


sub entnehme_ecke {
    #--------------------------------------------------------------------------
    # Methode      : e n t n e h m e _ e c k e
    # Klasse       : Graph
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Entfernt eine Ecke aus der Eckenmenge und gibt diese
    #                zurck.
    # Parameter    : Index
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 10.02.2004 - CD - erstellt
    #--------------------------------------------------------------------------

    return splice @{$_[0]->_ecken()}, $_[1], 1;

} # sub Graph::GED::Graph2D::entnehme_ecke


sub fuege_ecke_hinzu {
    #--------------------------------------------------------------------------
    # Methode      : f u e g e _ e c k e _ h i n z u
    # Klasse       : Graph
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Fgt eine Ecke zur Eckenmenge hinzu.
    # Parameter    : Ecke-Objekt
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 10.02.2004 - CD - erstellt
    #--------------------------------------------------------------------------

    push @{$_[0]->_ecken()}, $_[1];

} # sub Graph::GED::Graph2D::fuege_ecke_hinzu


sub x_von_a { return $_[0]->ecke($_[1]->a())->x() }
sub y_von_a { return $_[0]->ecke($_[1]->a())->y() }
sub x_von_e { return $_[0]->ecke($_[1]->e())->x() }
sub y_von_e { return $_[0]->ecke($_[1]->e())->y() }

sub x_von_a_index { return $_[0]->ecke($_[0]->kante($_[1])->a())->x() }
sub y_von_a_index { return $_[0]->ecke($_[0]->kante($_[1])->a())->y() }
sub x_von_e_index { return $_[0]->ecke($_[0]->kante($_[1])->e())->x() }
sub y_von_e_index { return $_[0]->ecke($_[0]->kante($_[1])->e())->y() }


#==============================================================================
#
#
#   M E T H O D E N
#
#
#==============================================================================


sub ecke_einfuegen {
    #--------------------------------------------------------------------------
    # Methode      : e c k e _ e i n f u e g e n
    # Klasse       : Graph::GED::Graph2D
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Fgt eine neue Ecke in den Graphen ein.
    # Parameter    : 1) X-Koordinate
    #                2) Y-Koordinate
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 08.02.2004 - CD - erstellt
    # 0.0.2 - 10.02.2004 - CD - neue Hilfsmethoden verwendet.
    #                           Umstellung auf die Eckenklasse.
    #--------------------------------------------------------------------------

    my $self = shift;

    die "Graph::GED::Graph2D::ecke_loeschen mit falscher Anzahl Parametern " .
        "aufgerufen!"
        if scalar @_ != 2;

    my $x = shift;
    my $y = shift;

    print "Graph::GED::Graph2D::ecke_einfuegen($x, $y)\n" if $Debug;

    #--------------------------------------------------------------------------
    # Basisklassenfunktionalitt aufrufen:
    #--------------------------------------------------------------------------
    $self->SUPER::ecke_einfuegen();

    #--------------------------------------------------------------------------
    # Koordinaten der Ecke im Raster ablegen:
    #--------------------------------------------------------------------------
    my $e = new Graph::GED::Ecke;
    $e->x($x);
    $e->y($y);
    $self->fuege_ecke_hinzu($e);

} # sub Graph::GED::Graph2D::ecke_einfuegen


sub ecke_loeschen {
    #--------------------------------------------------------------------------
    # Methode      : e c k e _ l o e s c h e n
    # Klasse       : Graph::GED::Graph2D
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Lscht die angegebene Ecke mit allen inzidierenden Kanten
    #                aus dem Graphen.
    # Parameter    : zu lschende Ecke
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 08.02.2004 - CD - erstellt
    # 0.0.2 - 10.02.2004 - CD - neue Hilfsmethoden verwendet.
    #                           Umstellung auf die Eckenklasse.
    #--------------------------------------------------------------------------

    my $self = shift;

    die "Graph::GED::Graph2D::ecke_loeschen mit falscher Anzahl Parametern " .
        "aufgerufen!"
        if scalar @_ != 1;
    die "Graph::GED::Graph2D::ecke_loeschen aufgerufen, aber der Graph hat " .
        "keine Ecken!"
        unless $self->n() > 0;

    my $e = shift;

    print "Graph::GED::Graph2D::ecke_loeschen($e)\n" if $Debug;

    #--------------------------------------------------------------------------
    # Basisklassenfunktionalitt aufrufen:
    #--------------------------------------------------------------------------
    $self->SUPER::ecke_loeschen($e);

    #--------------------------------------------------------------------------
    # Ecke lschen:
    #--------------------------------------------------------------------------
    $self->entnehme_ecke($e);
} # sub Graph::GED::Graph2D::ecke_loeschen


sub ecke_mergen {
    #--------------------------------------------------------------------------
    # Methode      : e c k e _ m e r g e n
    # Klasse       : Graph::GED::Graph2D
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Mergt die angegebene Ecke (d.h. die Ecke wird gelscht
    #                und die Kanten zusammengezogen.
    # Parameter    : Zu mergende Ecke
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 08.02.2004 - CD - erstellt
    # 0.0.2 - 10.02.2004 - CD - neue Hilfsmethoden verwendet.
    #                           Umstellung auf die Eckenklasse.
    #--------------------------------------------------------------------------

    my $self = shift;

    die "Graph::GED::Graph2D::ecke_mergen mit falscher Anzahl Parametern " .
        "aufgerufen!"
        if scalar @_ != 1;


    my $e = shift;

    print "Graph::GED::Graph2D::ecke_mergen($e)\n" if $Debug;

    #--------------------------------------------------------------------------
    # Basisklassenfunktionalitt aufrufen:
    #--------------------------------------------------------------------------
    $self->SUPER::ecke_mergen($e);


    #--------------------------------------------------------------------------
    # Ecke lschen:
    #--------------------------------------------------------------------------
    $self->entnehme_ecke($e);
} # sub Graph::GED::Graph2D::ecke_mergen


sub clear {
    #--------------------------------------------------------------------------
    # Methode      : c l e a r
    # Klasse       : Graph::GED::Graph2D
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Lscht alle Ecken und Kanten.
    # Parameter    : keine
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 09.02.2003 - CD - Erstellt
    # 0.0.2 - 10.02.2004 - CD - neue Hilfsmethoden verwendet.
    #                           Umstellung auf die Eckenklasse.
    # 0.0.3 - 01.01.2005 - CD - Standardwerte der Farben etc. einstellen.
    # 0.0.4 - 14.01.2005 - CD - Neues Attribut hintergrundbild
    # 0.0.5 - 16.01.2005 - CD - Neues Attribut kantendicke
    #                           Neues Attribut dateiname
    #--------------------------------------------------------------------------

    my $self = shift;

    die "Graph::GED::Graph2D::clear mit Parametern aufgerufen!"
        if scalar @_ > 0;

    print "Graph::GED::Graph2D::clear\n" if $Debug;

    #--------------------------------------------------------------------------
    # Ecken lschen:
    #--------------------------------------------------------------------------
    $self->loesche_ecken();

    #--------------------------------------------------------------------------
    # Basisfunktionalitt aufrufen:
    #--------------------------------------------------------------------------
    $self->SUPER::clear();

    #--------------------------------------------------------------------------
    # Standardwerte einstellen:
    #--------------------------------------------------------------------------
    $self->radius         (        3);
    $self->delta          (       -1);
    $self->eckenfarbe     ('#FF0000');
    $self->kantenfarbe    ('#000000');
    $self->hintergrund    ('#FFFFFF');
    $self->gitterzeigen   (        0);
    $self->gitterweite    (       25);
    $self->rastimgitter   (        0);
    $self->gitterfarbe    ('#C0C0C0');
    $self->hintergrundbild(       '');
    $self->kantendicke    (        1);
    $self->dateiname      (       '');
    $self->r2d();

} # sub Graph::GED::Graph2D::clear


sub ecke_zu_nah {
    #--------------------------------------------------------------------------
    # Methode      : e c k e _ z u _ n a h
    # Klasse       : Graph::GED::Graph2D
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Testet, ob die angegebenen Koordinaten einer schon vor-
    #                handenen Ecke zu nahe kommen.
    # Parameter    : x, y - zu testendes Koordinatenpaar
    # Rckgabewert : 0 oder 1 (1 meint Kollision)
    #--------------------------------------------------------------------------
    # 0.0.1 - 03.02.2003 - CD - Erstellt
    # 0.0.2 - 05.02.2004 - CD - Umstellung vieler globaler Variablen auf
    #                           %Konfig.
    #                           Umstellung vieler globaler Variablen auf
    #                           %Graph.
    # 0.0.3 - 08.02.2004 - CD - Neu als Klassenmethode
    # 0.0.4 - 10.02.2004 - CD - neue Hilfsmethoden verwendet.
    #                           Umstellung auf die Eckenklasse.
    #--------------------------------------------------------------------------

    my $self = shift;

    die "Graph::GED::Graph2D::ecke_zu_nah mit falscher Anzahl Parametern aufgerufen!"
        if scalar @_ != 2;

    my $x     = shift;
    my $y     = shift;

    print "Graph::GED::Graph2D::ecke_zu_nah, x='$x', y='$y', ",
          "delta='", $self->delta(), "'\n"
          if $Debug;

    die "Interner Fehler!" if scalar @{$self->_ecken()} != $self->n();

    for my $i (0..$self->n()-1) {
        my $ecke = $self->ecke($i);
        if (
            abs($ecke->x() - $x) < $self->delta() and
            abs($ecke->y() - $y) < $self->delta()
           )
        {
            print "Kollison mit Ecke $i auf (",$ecke->x(),", ",$ecke->y(),")\n"
                  if $Debug>1;
            return 1;
        }
    }

    return 0;
} # sub Graph::GED::Graph2D::ecke_zu_nah


sub ecke_zu_nah_ohne {
    #--------------------------------------------------------------------------
    # Methode      : e c k e _ z u _ n a h _ o h n e
    # Klasse       : Graph::GED::Graph2D
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Testet, ob die angegebenen Koordinaten einer schon vor-
    #                handenen Ecke zu nahe kommen, mit aunahme der angegebe-
    #                nen Ecke.
    # Parameter    : x, y - zu testendes Koordinatenpaar
    #                e    - Ecke
    # Rckgabewert : 0 oder 1 (1 meint Kollision)
    #--------------------------------------------------------------------------
    # 0.0.1 - 05.02.2003 - CD - Erstellt
    # 0.0.2 - 05.02.2004 - CD - Umstellung vieler globaler Variablen auf
    #                           %Graph.
    # 0.0.3 - 08.02.2004 - CD - Neu als Klassenmethode
    # 0.0.4 - 10.02.2004 - CD - neue Hilfsmethoden verwendet.
    #                           Umstellung auf die Eckenklasse.
    #--------------------------------------------------------------------------

    my $self = shift;

    die "Graph::GED::Graph2D::ecke_zu_nah_ohne mit falscher Anzahl Parametern aufgerufen!"
        if scalar @_ != 3;

    my $x     = shift;
    my $y     = shift;
    my $e     = shift;

    print "Graph::GED::Graph2D::ecke_zu_nah_ohne, x='$x', y='$y', Ecke = '$e', ",
          "delta='", $self->delta(), "'\n"
          if $Debug;

    die "Interner Fehler!" if scalar @{$self->_ecken()} != $self->n();

    for my $i (0..$self->n()-1) {
        next if $i == $e;
        my $ecke = $self->ecke($i);
        if (
            abs($ecke->x() - $x) < $self->delta() and
            abs($ecke->y() - $y) < $self->delta()
           )
        {
            print "Kollison mit Ecke $i auf (",$ecke->x(),", ",$ecke->y(),")\n"
                  if $Debug>1;
            return 1;
        }
    }

    return 0;
} # sub Graph::GED::Graph2D::ecke_zu_nah_ohne


sub meint_ecke {
    #--------------------------------------------------------------------------
    # Methode      : m e i n t _ e c k e
    # Klasse       : Graph::GED::Graph2D
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Es wird die Ecke zu den Koordinaten herausgesucht, die am
    #                nchsten liegt. Dabei darf nicht zu weit entfernt geklickt
    #                werden.
    # Parameter    : x, y
    # Rckgabewert : Eckennummer oder -1, wenn keine Ecke in der Nhe liegt.
    #--------------------------------------------------------------------------
    # 0.0.1 - 03.02.2003 - CD - Erstellt
    # 0.0.2 - 05.02.2004 - CD - Umstellung vieler globaler Variablen auf %Konfig.
    #                           Umstellung vieler globaler Variablen auf %Graph.
    # 0.0.3 - 08.02.2004 - CD - Neu als Klassenmethode
    # 0.0.4 - 10.02.2004 - CD - neue Hilfsmethoden verwendet.
    #                           Umstellung auf die Eckenklasse.
    # 0.0.5 - 01.01.2005 - CD - Bercksichtigung eventueller individueller
    #                           Radien.
    # 0.0.6 - 03.01.2005 - CD - radius2delta zur Berechnung verwendet.
    #--------------------------------------------------------------------------

    my $self = shift;

    die "Graph::GED::Graph2D::meint_ecke mit falscher Anzahl Parametern aufgerufen!"
        if scalar @_ != 2;


    my $x     = shift;
    my $y     = shift;


    print "Graph::GED::Graph2D::meint_ecke, x='$x', y='$y', ",
          "delta='", $self->delta(), "'\n"
          if $Debug;

    my $e     = -1;


    die "Interner Fehler!" if scalar @{$self->_ecken()} != $self->n();

    for my $i (0..$self->n()-1) {
        my $ecke   = $self->ecke($i);
        my $radius = $ecke->radius() || $self->radius();
        my $delta  = $self->radius2delta($radius);
        if (
            abs($ecke->x() - $x) < $delta and
            abs($ecke->y() - $y) < $delta
           )
        {
            print "Ecke $i auf (", $ecke->x(), ", ", $ecke->y(),
                  ") ist gemeint.\n"
                  if $Debug>1;
            $e = $i;
            last;
        }
    }


    return $e;

} # sub Graph::GED::Graph2D::meint_ecke


sub meint_kante {
    #--------------------------------------------------------------------------
    # Methode      : m e i n t _ k a n t e
    # Klasse       : Graph::GED::Graph2D
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Es wird die Kante zu den Koordinaten herausgesucht, die
    #                am nchsten liegt. Dabei darf nicht zu weit entfernt ge-
    #                klickt werden.
    # Parameter    : x, y
    # Rckgabewert : Kantennummer oder -1, wenn keine Ecke in der Nhe liegt.
    #                Falls eine Liste erwartet wird, werden alle in der Nhe
    #                befindlichen Kantenindizes als Liste zurck gegeben.
    #--------------------------------------------------------------------------
    # 0.0.1 - 03.02.2003 - CD - Erstellt
    # 0.0.2 - 10.02.2003 - CD - Mit Funktion gefllt.
    # 0.0.3 - 05.02.2004 - CD - Umstellung vieler globaler Variablen auf
    #                           %Konfig.
    #                           Umstellung vieler globaler Variablen auf
    #                           %Graph.
    # 0.0.4 - 08.02.2004 - CD - Neu als Klassenmethode
    # 0.0.5 - 10.02.2004 - CD - neue Hilfsmethoden verwendet.
    #                           Umstellung auf die Kantenklasse.
    #                           Umstellung auf die Eckenklasse.
    # 0.0.6 - 12.02.2004 - CD - Neue Hilfsmethoden x_von_a etc. verwendet.
    # 0.0.7 - 14.01.2005 - CD - Alle Kanten die gemeint sein knnten fangen,
    #                           falls eine Liste erwartet wird (wantarray).
    #--------------------------------------------------------------------------

    my $self = shift;

    die "Graph::GED::Graph2D::meint_kante mit falscher Anzahl Parametern aufgerufen!"
        if scalar @_ != 2;


    my $x     = shift;
    my $y     = shift;

    print "Graph::GED::Graph2D::meint_kante, x='$x', y='$y', ",
          "delta='", $self->delta(), "'\n"
          if $Debug;


    die "Interner Fehler!" if scalar @{$self->_kanten()} != $self->m();

    my @kanten; # gemeinte Kantenmenge

    #--------------------------------------------------------------------------
    # Schleife ber alle Kanten:
    #--------------------------------------------------------------------------
    for my $i (0..$self->m()-1) {
        my $kante = $self->kante($i);
        my $ax = $self->x_von_a($kante);
        my $ay = $self->y_von_a($kante);
        my $ex = $self->x_von_e($kante);
        my $ey = $self->y_von_e($kante);

        my $abs = Graph::GED::Geom::abstand_punkt_linie($x, $y, $ax, $ay, $ex, $ey);

        if ($abs < $self->delta()) {
            push @kanten, $i;
            if (@kanten > 1 and not wantarray) {
                print "Uneindeutig! Kante $kanten[0] und Kante $kanten[1] ",
                      "sind nah dran.\n"
                      if $Debug;
                return -1;
            }
            else {
                print "Kante $i (",
                      $kante->a(), ", ", $kante->e(),
                      ") ist gemeint.\n",
                      "[Abstand ist $abs]\n"
                      if $Debug;
            }
        }
    }

    if (wantarray) {
        return @kanten;
    }
    else {
        if (@kanten > 0) {
            return $kanten[0];
        }
        else {
            return -1;
        }
    }
} # sub Graph::GED::Graph2D::meint_kante


sub r2d {
    #--------------------------------------------------------------------------
    # Methode      : r 2 d
    # Klasse       : Graph::GED::Graph2D
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Berechnet Delta aus dem Radius.
    #                In Funktion verschoben, falls Radius gendert wird, was
    #                geplant ist.
    # Parameter    : keine
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 03.02.2003 - CD - Erstellt
    # 0.0.2 - 05.02.2004 - CD - Umstellung vieler globaler Variablen auf
    #                           %Konfig.
    #                           Umstellung vieler globaler Variablen auf
    #                           %Graph.
    # 0.0.3 - 08.02.2004 - CD - Neu als Klassenmethode
    # 0.0.4 - 02.01.2005 - CD - andere Berechnung
    # 0.0.5 - 03.01.2005 - CD - Berechnung in Methode radius2delta verlagert.
    #--------------------------------------------------------------------------

    my $self = shift;

    die "Graph::GED::Graph2D::r2d mit Parametern aufgerufen!"
        if scalar @_ > 0;


    print "Graph::GED::Graph2D::r2d: Radius = '", $self->radius(),
          "', Delta-Alt = '", $self->delta(), "', "
          if $Debug > 1;

    #$self->delta(2 * $self->radius() + $self->radius() + 2);
    #$self->delta($self->radius() + 5);
    $self->delta($self->radius2delta($self->radius()));

    print "Delta-Neu = '", $self->delta(), "'\n" if $Debug > 1;
} # sub Graph::GED::Graph2D::r2d


sub radius2delta {
    #--------------------------------------------------------------------------
    # sub          : r a d i u s 2 d e l t a
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Berechnet aus dem Eckenradius den Abstand delta, um den
    #                herum ein Klick als zu einer Ecke gehrig gewertet wird.
    # Parameter    : Radius
    # Rckgabewert : Delta
    #--------------------------------------------------------------------------
    # 0.0.1 - 03.01.2005 - CD - Erstellt
    #--------------------------------------------------------------------------

    my ($self, $radius) = @_;
    return $radius + 5;

} # sub Graph::GED::Graph2D::radius2delta


sub struktur_einfuegen {
    #--------------------------------------------------------------------------
    # Methode      : s t r u k t u r _ e i n f u e g e n
    # Klasse       : Graph::GED::Graph2D
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Fgt Kreise / Vollstndige Graphen ein
    # Parameter    : Strukturkennung, beginnt mt C ode K gefolgt von einer
    #                natrlichen Zahl > 2.
    #                Oder 'M' fr den Mittelpunkt.
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 11.02.2003 - CD - Erstellt
    # 0.0.2 - 12.02.2003 - CD - Koordinatenberechnung korrigiert.
    #                           Mittelpunkt einzeichenbar.
    # 0.0.3 - 18.03.2003 - CD - Neue Linientypkrzel verwendet.
    # 0.0.4 - 05.02.2004 - CD - Umstellung vieler globaler Variablen auf
    #                           %Konfig.
    #                           Umstellung vieler globaler Variablen auf
    #                           %Graph.
    # 0.0.5 - 05.02.2004 - CD - canvas_groesse() aufgerufen zur Ermittlung der
    #                           Gre der Zeichenflche.
    # 0.0.6 - 07.02.2004 - CD - Radius wird an zurckgelieferte Breite ange-
    #                           passt.
    # 0.0.7 - 08.02.2004 - CD - Radius vergrert.
    # 0.0.8 - 09.02.2004 - CD - Neu als Klassenmethode. Die Klassenvariablen
    #                           $MinX, $MinY, $MaxX, $MaxY werden bei jeder
    #                           Grennderung der Zeichenflche im aufrufen-
    #                           den Programm gendert!
    #                           Auerdem wurden die Methoden zum Einfgen
    #                           von Ecken und Kanten verwendet.
    #--------------------------------------------------------------------------

    my $self = shift;

    die "Graph::GED::Graph2D::struktur_einfuegen mit falscher Anzahl Parametern aufgerufen!"
        if scalar @_ != 1;

    my $struktur = shift;                # 'C3', ...
    my $pi       = 3.141592653589793238; # Kreiszahl PI (abgeschnitten...)
    my $offx     = int($MaxX/2); # Mittelpunkt des Kreises, x-Koord.
    my $offy     = int($MaxY/2); # Mittelpunkt des Kreises, y-Koord.
    my $radius   = ($MaxX<=$MaxY?$MaxX:$MaxY) / 2 - 30;

    print "Graph::GED::Graph2D::struktur_einfuegen, Struktur = '$struktur'\n"
          if $Debug;


    #--------------------------------------------------------------------------
    # Mittelpunkt:
    #--------------------------------------------------------------------------
    if (lc($struktur) eq 'm') {
        $self->ecke_einfuegen($offx, $offy);
        return;
    }

    #--------------------------------------------------------------------------
    # Kreis oder vollstndiger Graph:
    #--------------------------------------------------------------------------
    $struktur =~ /^(\w)(\d+)$/;

    my $c_oder_k = $1;
    my $zahl     = $2;

    $c_oder_k = '' unless defined $c_oder_k;
    $zahl     = 0  unless defined $zahl;

    print "Eingefgt werden soll ein ",
          lc($c_oder_k) eq 'c' ? "Kreis" : "Vollstndiger Graph",
          " mit $zahl Ecken.\n"
          if $Debug;

    die "Falscher Aufruf von struktur_einfgen, Parameter war '$struktur'!"
        unless (lc($c_oder_k) eq 'c' or lc($c_oder_k) eq 'k') and
               $zahl > 2 and
               $zahl < 100;

    my $startecke = $self->n();
    my $endecke   = $self->n() + $zahl - 1;

    for my $i ($startecke .. $endecke) {
        #----------------------------------------------------------------------
        # Eckenkoordinaten festlegen und Ecke einfgen:
        #----------------------------------------------------------------------
        my $n      = 1 + $i - $startecke;  # Nummer der aktuell zu setzenden
                                           # Ecke, beginnend mit 1
        my $phi    = ($n-1)*(2*$pi)/$zahl; # Winkel zwischen der Linie vom
                                           # Mittelpunkt zur aktuellen Ecke
                                           # und der x-Achse.
        my $nx     = $radius * cos $phi;   # x-Koordinate der aktuellen Ecke
        my $ny     = $radius * sin $phi;   # y-Koordinate der aktuellen Ecke

        $self->ecke_einfuegen(int($nx+$offx+.5), int($ny+$offy+.5));
    }

    for my $i ($startecke .. $endecke) {
        #----------------------------------------------------------------------
        # Kreiskanten einziehen:
        #----------------------------------------------------------------------
        if (lc($c_oder_k) eq 'c') {
            my $j = $i==$endecke ? $startecke : $i+1;
            $self->kante_einfuegen($i, $j);
        }
        #----------------------------------------------------------------------
        # Alle Kanten einziehen:
        # (aber nicht doppelt, deshalb erst ab $i+1)
        #----------------------------------------------------------------------
        elsif (lc($c_oder_k) eq 'k') {
            for my $j ($i+1 .. $endecke) {
                $self->kante_einfuegen($i, $j);
            }
        }
    }
} # sub Graph::GED::Graph2D::struktur_einfuegen


sub erzeuge_liniengraphen  {
    #--------------------------------------------------------------------------
    # Methode      : e r z e u g e _ l i n i e n g r a p h e n
    # Klasse       : Graph::GED::Graph2D
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Erzeugt aus dem gegebenen Graphen seinen Liniengraphen.
    # Parameter    : keine
    # Rckgabewert : keiner (ein neuer Graph wird erzeugt)
    #--------------------------------------------------------------------------
    # 0.0.1 - 06.02.2004 - CD - Erstellt
    # 0.0.2 - 08.02.2004 - CD - Neu als Klassenmethode
    # 0.0.3 - 10.02.2004 - CD - neue Hilfsmethoden verwendet.
    #                           Umstellung auf die Kantenklasse.
    #                           Umstellung auf die Eckenklasse.
    # 0.0.4 - 12.02.2004 - CD - Neue Hilfsmethoden x_von_a etc. verwendet.
    #--------------------------------------------------------------------------

    my $self = shift;

    die "Graph::GED::Graph2D::r2d mit Parametern aufgerufen!"
        if scalar @_ > 0;

    print "Graph::GED::Graph2D::erzeuge_liniengraphen\n" if $Debug;

    die "Interner Fehler!" if scalar @{$self->_kanten()} != $self->m();

    #--------------------------------------------------------------------------
    # Graph hat keine Kanten:
    #--------------------------------------------------------------------------
    if ($self->m() == 0) {
        $self->clear();
    }

    #--------------------------------------------------------------------------
    # Graph hat Kanten:
    #--------------------------------------------------------------------------
    else {
        my @ecken  = (); # neue Eckenmenge
        my @kanten = (); # neue Kantenmenge
        my $n      = $self->m();
        my $m      = 0;

        #----------------------------------------------------------------------
        # Neue Ecken berechnen:
        #----------------------------------------------------------------------
        for my $k (0..$self->m()-1) {
            my $ecke  = new Graph::GED::Ecke;
            $ecke->x( int(($self->x_von_a_index($k) +
                           $self->x_von_e_index($k)) / 2) );
            $ecke->y( int(($self->y_von_a_index($k) +
                           $self->y_von_e_index($k)) / 2) );
            push @ecken, $ecke;
        }

        #----------------------------------------------------------------------
        # Neue Kanten berechnen:
        #----------------------------------------------------------------------
        for my $k1 (0..$self->m()-2) {
            for my $k2 ($k1+1..$self->m()-1) {
                my $kante1 = $self->kante($k1);
                my $kante2 = $self->kante($k2);
                if (
                    $kante1->a() == $kante2->a() or
                    $kante1->a() == $kante2->e() or
                    $kante1->e() == $kante2->a() or
                    $kante1->e() == $kante2->e()
                   )
                {
                    my $kante = new Graph::GED::Kante;
                    $kante->a($k1);
                    $kante->e($k2);
                    $kante->form('k_ins');
                    push @kanten, $kante;
                    ++$m;
                }
            }
        }

        #----------------------------------------------------------------------
        # Graph lschen:
        #----------------------------------------------------------------------
        $self->clear();

        #----------------------------------------------------------------------
        # Liniengraphendaten speichern:
        #----------------------------------------------------------------------
        $self->setze_ecken (\@ecken );
        $self->setze_kanten(\@kanten);

        $self->n($n);
        $self->m($m);

        #----------------------------------------------------------------------
        # Weitere Grapheneigenschaften zurcksetzen:
        #----------------------------------------------------------------------
        $self->kantenform('');
    }
} # sub Graph::GED::Graph2D::erzeuge_liniengraphen


sub toLaTeX ($;$$$) {
    #--------------------------------------------------------------------------
    # Methode      : t o L a T e X
    # Klasse       : Graph::GED::Graph2D
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Gibt den selfen in Form einer LaTeX-Datei aus.
    # Parameter    : dname      - Der Dateiname. Und optional:
    #                faktor     - Mastabsvergrerung oder - verkleinerung.
    #                xmax, ymax - Bestimmen zusammen mit dem Faktor die Gre
    #                             (bei -1 werden diese Gren automatisch be-
    #                             rechnet).
    #                Es muss der Name angegeben werden, die anderen Parameter
    #                knnen weggelassen werden, dann werden sie mit 1 bzw. -1
    #                initialisiert.
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 13.02.2003 - CD - Erstellt
    # 0.0.2 - 05.02.2004 - CD - Umstellung vieler globaler Variablen auf
    #                           %Konfig.
    #                           Umstellung vieler globaler Variablen auf
    #                           %self.
    # 0.0.3 - 09.02.2004 - CD - Umstellungen auf die Verwendung von self2D.
    # 0.0.4 - 10.02.2004 - CD - Umstellung auf die Kantenklasse.
    #                           Umstellung auf die Eckenklasse.
    # 0.0.5 - 12.02.2004 - CD - Neue Hilfsmethoden x_von_a etc. verwendet.
    # 0.0.6 - 16.01.2005 - CD - Nach Graph2D verschoben.
    #--------------------------------------------------------------------------

    my $self   = shift;
    my $dname  = shift;
    my $faktor = shift;
    my $xmax   = shift;
    my $ymax   = shift;


    die "toLaTeX ohne Dateinamen aufgerufen" unless defined $dname;
    $faktor =  1                             unless defined $faktor;
    $xmax   = -1                             unless defined $xmax;
    $ymax   = -1                             unless defined $ymax;

    #--------------------------------------------------------------------------
    # xmax und ymax berechnen, falls gewnscht:
    #--------------------------------------------------------------------------
    if ($xmax == -1) {
        for my $i (0..$self->n()-1) {
            $xmax = $self->ecke($i)->x() if $xmax < $self->ecke($i)->x();
        }
    }
    if ($ymax == -1) {
        for my $i (0..$self->n()-1) {
            $ymax = $self->ecke($i)->y() if $ymax < $self->ecke($i)->y();  #;
        }
    }
    # Eventuell xmin, ymin berechnen, von allen Werten diese subtrahieren!
    # Aber nur bei Automatikberechnung!

    #--------------------------------------------------------------------------
    # Ankndigung der folgenden Aktion:
    #--------------------------------------------------------------------------
    print "Speichere selfen als LaTeXfile $dname ...\n" if $Debug;

    #--------------------------------------------------------------------------
    # Eigentliches Speichern:
    #--------------------------------------------------------------------------
    open(LOUT, ">$dname") or
        die "Kann Datei '$dname' nicht zum Schreiben ffnen: $!";

    print LOUT "%% Dies ist eine von GED automatisch erzeugte Datei!\n",
               "%% self: $dname\n",
               "\\unitlength 1.00mm\n",
               "\\begin{picture}(", $xmax*$faktor, ',', $ymax*$faktor, ")\n";

    # Ecken ausgeben:
    for my $i (0..$self->n()-1) {
        my $x = $self->ecke($i)->x() * $faktor;
        my $y = $self->ecke($i)->y() * $faktor;                             #*
        my $n = 'Bezeichnung'; # $self->EckenName()->[$i] spter vielleicht ...

        # Ecke:
        print LOUT "    \\put(", $x, ',', $y, "){\\circle*{1.5}}\n";
        # Eckennamen:
        print LOUT "    \\put(", (1.0 + $x), ',', ($y - 2.5), ")",
                   "{\\makebox(0,0)[lb]{{\\small ", $n, "}}}\n";
    }

    # Kanten ausgeben:
    for my $i (0..$self->m()-1) {
        my $x1 = $self->x_von_a_index($i) * $faktor;
        my $y1 = $self->y_von_a_index($i) * $faktor;
        my $x2 = $self->x_von_e_index($i) * $faktor;
        my $y2 = $self->y_von_e_index($i) * $faktor;

        my $dx   = $x2 - $x1;
        my $dy   = $y2 - $y1;
        my $dist = sqrt($dx*$dx + $dy*$dy);
        my $wdh = int(5 * $dist);

        $dx = $dx / $wdh;
        $dy = $dy / $wdh;

        print LOUT "    \\multiput(", $x1, ",", $y1, ")(", $dx, ",", $dy, ")",
                   "{", $wdh, "}{\\circle*{0.1}}\n";
    }

    print LOUT "\\end{picture}\n";

    close LOUT or die "Kann Datei '$dname' nicht schlieen: $!";
} # sub Graph::GED::Graph2D::toLaTeX


sub nach_links () {
    #--------------------------------------------------------------------------
    # Methode      : n a c h _ l i n k s
    # Klasse       : Graph::GED::Graph2D
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Verschiebt den Graphen nach links.
    # Parameter    : Dateiname
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 16.01.2005 - CD - Erstellt
    # 0.0.2 - 17.01.2005 - CD - Nach Graph2D verschoben.
    #--------------------------------------------------------------------------

    my $self = shift;

    for my $e (@{ $self->_ecken() }) {
        $e->x($e->x() - $self->gitterweite());
    }
} # sub Graph::GED::Graph2D::nach_links


sub nach_rechts () {
    #--------------------------------------------------------------------------
    # Methode      : n a c h _ r e c h t s
    # Klasse       : Graph::GED::Graph2D
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Verschiebt den Graphen nach rechts.
    # Parameter    : Dateiname
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 16.01.2005 - CD - Erstellt
    # 0.0.2 - 17.01.2005 - CD - Nach Graph2D verschoben.
    #--------------------------------------------------------------------------

    my $self = shift;

    for my $e (@{ $self->_ecken() }) {
        $e->x($e->x() + $self->gitterweite());
    }
} # sub Graph::GED::Graph2D::nach_rechts


sub nach_oben () {
    #--------------------------------------------------------------------------
    # Methode      : n a c h _ o b e n
    # Klasse       : Graph::GED::Graph2D
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Verschiebt den Graphen nach oben.
    # Parameter    : Dateiname
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 16.01.2005 - CD - Erstellt
    # 0.0.2 - 17.01.2005 - CD - Nach Graph2D verschoben.
    #--------------------------------------------------------------------------

    my $self = shift;

    for my $e (@{ $self->_ecken() }) {
        $e->y($e->y() - $self->gitterweite());
    }
} # sub Graph::GED::Graph2D::nach_oben


sub nach_unten () {
    #--------------------------------------------------------------------------
    # Methode      : n a c h _ u n t e n
    # Klasse       : Graph::GED::Graph2D
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Verschiebt den Graphen nach unten.
    # Parameter    : Dateiname
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 16.01.2005 - CD - Erstellt
    # 0.0.2 - 17.01.2005 - CD - Nach Graph2D verschoben.
    #--------------------------------------------------------------------------

    my $self = shift;

    for my $e (@{ $self->_ecken() }) {
        $e->y($e->y() + $self->gitterweite());
    }
} # sub Graph::GED::Graph2D::nach_unten


sub zentrieren () {
    #--------------------------------------------------------------------------
    # Methode      : z e n t r i e r e n
    # Klasse       : Graph::GED::Graph2D
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Zentriert den Graphen.
    #
    #                (Die Klassenvariablen $MinX, $MinY, $MaxX, $MaxY werden
    #                bei jeder Grennderung der Zeichenflche im aufrufen-
    #                den Programm in refresh() gendert!)
    # Parameter    : Dateiname
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 17.01.2005 - CD - Erstellt
    #--------------------------------------------------------------------------

    my $self = shift;

    return unless $self->n() > 0;

    #
    # Extremale Koordinaten des Graphen bestimmen:
    #
    my $minx = my $maxx = $self->_ecken()->[0]->x();
    my $miny = my $maxy = $self->_ecken()->[0]->y();
    for my $e (@{ $self->_ecken() }) {
        $minx = $e->x() if $e->x() < $minx;
        $maxx = $e->x() if $e->x() > $maxx;
        $miny = $e->y() if $e->y() < $miny;
        $maxy = $e->y() if $e->y() > $maxy;
    }

    #
    # Mittelpunktkoordinate berechnen:
    #
    my $centerx = int(($minx + $maxx) / 2);
    my $centery = int(($miny + $maxy) / 2);

    #
    # Koordinatentransformation:
    #
    my $xsummand = int(($MaxX + $MinX) / 2) - $centerx;
    my $ysummand = int(($MaxY + $MinY) / 2) - $centery;
    for my $e (@{ $self->_ecken() }) {
        $e->x( $e->x() + $xsummand );
        $e->y( $e->y() + $ysummand );
    }
} # sub Graph::GED::Graph2D::zentrieren


sub zentrieren_skalieren () {
    #--------------------------------------------------------------------------
    # Methode      : s a v e
    # Klasse       : Graph::GED::Graph2D
    #--------------------------------------------------------------------------
    # sub          : z e n t r i e r e n _ s k a l i e r e n
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Zentriert den Graphen und skaliert ihn auf die sichtbare
    #                Arbeitsflche der Canvas.
    #
    #                (Die Klassenvariablen $MinX, $MinY, $MaxX, $MaxY werden
    #                bei jeder Grennderung der Zeichenflche im aufrufen-
    #                den Programm in refresh() gendert!)
    # Parameter    : Dateiname
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 17.01.2005 - CD - Erstellt
    #--------------------------------------------------------------------------

    my $self = shift;

    return unless $self->n() > 0;

    #
    # Extreme erlaubte Werte bestimmen:
    #
    my $sminx = $MinX + 20;
    my $sminy = $MinY + 20;
    my $smaxx = $MaxX - 20;
    my $smaxy = $MaxY - 20;

    #
    # Extremale Koordinaten des Graphen bestimmen:
    #
    my $minx = my $maxx = $self->_ecken()->[0]->x();
    my $miny = my $maxy = $self->_ecken()->[0]->y();
    for my $e (@{ $self->_ecken() }) {
        $minx = $e->x() if $e->x() < $minx;
        $maxx = $e->x() if $e->x() > $maxx;
        $miny = $e->y() if $e->y() < $miny;
        $maxy = $e->y() if $e->y() > $maxy;
    }

    #
    # Koordinatentransformation:
    #
    my $xfaktor = ($smaxx-$sminx) / ($maxx-$minx);
    my $yfaktor = ($smaxy-$sminy) / ($maxy-$miny);
    for my $e (@{ $self->_ecken() }) {
        $e->x( $sminx + int( ($e->x() - $minx) * $xfaktor) );
        $e->y( $sminy + int( ($e->y() - $miny) * $yfaktor) );
    }

    #
    # Gitter mit verndern:
    #
    my $faktor = $xfaktor > $yfaktor ? $xfaktor : $yfaktor;
    $self->gitterweite(int($self->gitterweite() * $faktor));
    if ($self->rastimgitter()) {
        # spter Gitter einrasten, wenn Gitterklasse erstellt...
    }
} # sub Graph::GED::Graph2D::zentrieren_skalieren


sub kleiner () {
    #--------------------------------------------------------------------------
    # Methode      : k l e i n e r
    # Klasse       : Graph::GED::Graph2D
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Verkleinert den Graphen.
    #
    #                (Die Klassenvariablen $MinX, $MinY, $MaxX, $MaxY werden
    #                bei jeder Grennderung der Zeichenflche im aufrufen-
    #                den Programm in refresh() gendert!)
    # Parameter    : Dateiname
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 17.01.2005 - CD - Erstellt
    #--------------------------------------------------------------------------

    my $self = shift;

    return unless $self->n() > 0;

    #
    # Extremale Koordinaten des Graphen bestimmen:
    #
    my $minx = my $maxx = $self->_ecken()->[0]->x();
    my $miny = my $maxy = $self->_ecken()->[0]->y();
    for my $e (@{ $self->_ecken() }) {
        $minx = $e->x() if $e->x() < $minx;
        $maxx = $e->x() if $e->x() > $maxx;
        $miny = $e->y() if $e->y() < $miny;
        $maxy = $e->y() if $e->y() > $maxy;
    }

    #
    # Mittelpunktkoordinate berechnen:
    #
    my $centerx = int(($minx + $maxx) / 2);
    my $centery = int(($miny + $maxy) / 2);

    #
    # Koordinatentransformation:
    #
    #my $faktor = 0.66666;
    my $faktor = 0.8;
    for my $e (@{ $self->_ecken() }) {
        $e->x( $centerx + int( ($e->x() - $centerx) * $faktor ) );
        $e->y( $centery + int( ($e->y() - $centery) * $faktor ) );
    }

    #
    # Gitter mit verndern:
    #
    $self->gitterweite(int($self->gitterweite() * $faktor));
    if ($self->rastimgitter()) {
        # spter Gitter einrasten, wenn Gitterklasse erstellt...
    }
} # sub Graph::GED::Graph2D::kleiner


sub groesser () {
    #--------------------------------------------------------------------------
    # Methode      : g r o e s s e r
    # Klasse       : Graph::GED::Graph2D
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Vergrert den Graphen.
    #
    #                (Die Klassenvariablen $MinX, $MinY, $MaxX, $MaxY werden
    #                bei jeder Grennderung der Zeichenflche im aufrufen-
    #                den Programm in refresh() gendert!)
    # Parameter    : Dateiname
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 17.01.2005 - CD - Erstellt
    #--------------------------------------------------------------------------

    my $self = shift;

    return unless $self->n() > 0;

    #
    # Extremale Koordinaten des Graphen bestimmen:
    #
    my $minx = my $maxx = $self->_ecken()->[0]->x();
    my $miny = my $maxy = $self->_ecken()->[0]->y();
    for my $e (@{ $self->_ecken() }) {
        $minx = $e->x() if $e->x() < $minx;
        $maxx = $e->x() if $e->x() > $maxx;
        $miny = $e->y() if $e->y() < $miny;
        $maxy = $e->y() if $e->y() > $maxy;
    }

    #
    # Mittelpunktkoordinate berechnen:
    #
    my $centerx = int(($minx + $maxx) / 2);
    my $centery = int(($miny + $maxy) / 2);

    #
    # Koordinatentransformation:
    #
    #my $faktor = 1.5;
    my $faktor = 1.25;
    for my $e (@{ $self->_ecken() }) {
        $e->x( $centerx + int( ($e->x() - $centerx) * $faktor ) );
        $e->y( $centery + int( ($e->y() - $centery) * $faktor ) );
    }

    #
    # Gitter mit verndern:
    #
    $self->gitterweite(int($self->gitterweite() * $faktor));
    if ($self->rastimgitter()) {
        # spter Gitter einrasten, wenn Gitterklasse erstellt...
    }
} # sub Graph::GED::Graph2D::groesser


sub save {
    #--------------------------------------------------------------------------
    # Methode      : s a v e
    # Klasse       : Graph::GED::Graph2D
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Speichert den Graphen, Formatermittlung erfolgt ber den
    #                Dateinamen.
    # Parameter    : Dateiname
    # Rckgabewert : 1 : Format wurde ermittelt und der Graph gespeichert.
    #                0 : Format wurde nicht ermittelt und nicht gespeichert.
    #--------------------------------------------------------------------------
    # 0.0.1 - 16.01.2005 - CD - Erstellt
    # 0.0.2 - 16.01.2005 - CD - ged3 eingebaut
    #--------------------------------------------------------------------------

    my ($self, $dateiname) = @_;


    if ($dateiname =~ /\.ged$/) {
        $self->save_ged($dateiname);  # Graph im ged-Format speichern
    }
    elsif ($dateiname =~ /\.ged2$/) {
        $self->save_ged2($dateiname); # Graph im ged2-Format speichern
    }
    elsif ($dateiname =~ /\.ged3$/) {
        $self->save_ged3($dateiname); # Graph im ged3-Format speichern
    }
    else {
        print "UNBEKANNTES LADEFORMAT ($dateiname)\n";
        print "\a";
        return 0;
    }

    $self->dateiname($dateiname);

    return 1;
} # sub Graph::GED::Graph2D::save


sub load {
    #--------------------------------------------------------------------------
    # Methode      : l o a d
    # Klasse       : Graph::GED::Graph2D
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Ldt den Graphen, Formatermittlung erfolgt ber den
    #                Dateinamen.
    # Parameter    : Dateiname
    # Rckgabewert : 1 : Format wurde ermittelt und der Graph geladen.
    #                0 : Format wurde nicht ermittelt und nicht geladen.
    #--------------------------------------------------------------------------
    # 0.0.1 - 01.01.2005 - CD - Erstellt
    # 0.0.2 - 16.01.2005 - CD - ged3 eingebaut
    #--------------------------------------------------------------------------

    my ($self, $dateiname) = @_;


    if ($dateiname =~ /\.ged$/) {
        $self->load_ged($dateiname);  # Graph im ged-Format laden
    }
    elsif ($dateiname =~ /\.ged2$/) {
        $self->load_ged2($dateiname); # Graph im ged2-Format laden
    }
    elsif ($dateiname =~ /\.ged3$/) {
        $self->load_ged3($dateiname); # Graph im ged3-Format laden
    }
    else {
        print "UNBEKANNTES LADEFORMAT ($dateiname)\n";
        print "\a";
        return 0;
    }

    $self->dateiname($dateiname);

    return 1;
} # sub Graph::GED::Graph2D::load


sub save_ged {
    #--------------------------------------------------------------------------
    # Methode      : s a v e _ g e d
    # Klasse       : Graph::GED::Graph2D
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Speichert den Graphen unter dem gegebenen Dateinamen im
    #                ged-Format.
    # Parameter    : Dateiname
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 01.01.2005 - CD - Erstellt
    #--------------------------------------------------------------------------

    my $self = shift;

    die "Graph::GED::Graph2D::save_ged mit falscher Anzahl Parametern aufgerufen!"
        if scalar @_ != 1;

    my $dateiname = shift;                # Dateiname

    open (OUT, ">$dateiname") or
        die "Kann '$dateiname' nicht zum Schreiben ffnen: $!";

    print OUT <<'HERE';
# Anzahl Ecken
# x-Koordinate der ersten Ecke;y-Koordinate der ersten Ecke
# x-Koordinate der zweiten Ecke;y-Koordinate der zweiten Ecke
# ...
# x-Koordinate der letzten Ecke;y-Koordinate der letzten Ecke
# Anzahl Kanten
# Anfangsecke der ersten Kante;Endecke der ersten Kante;Form der ertsen Kante
# Anfangsecke der zweiten Kante;Endecke der zweiten Kante;Form der zweiten Kante
# ...
# Anfangsecke der letzten Kante;Endecke der letzten Kante;Form der letzten Kante
HERE
    print OUT $self->n(), "\n";
    for my $i (0..$self->n-1) {
        print OUT $self->ecke($i)->x(), ";",
                  $self->ecke($i)->y(), "\n";                               #,
    }

    print OUT $self->m(), "\n";
    for my $i (0..$self->m()-1) {
        print OUT $self->kante($i)->a(),    ";",
                  $self->kante($i)->e(),    ";",
                  $self->kante($i)->form(), "\n";
    }

    close OUT or die "Kann '$dateiname' nicht schlieen: $!";

} # sub Graph::GED::Graph2D::save_ged


sub load_ged {
    #--------------------------------------------------------------------------
    # Methode      : l o a d _ g e d
    # Klasse       : Graph::GED::Graph2D
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Ldt den Graphen unter dem gegebenen Dateinamen im
    #                ged-Format.
    # Parameter    : Dateiname
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 01.01.2005 - CD - Erstellt
    #--------------------------------------------------------------------------

    my $self = shift;

    die "Graph::GED::Graph2D::load_ged mit falscher Anzahl Parametern aufgerufen!"
        if scalar @_ != 1;

    my $dateiname = shift;                # Dateiname

    $self->clear();

    open (IN, "<$dateiname") or
        die "Kann '$dateiname' nicht zum Lesen ffnen: $!";

    my $l = '';


    # Kommentarkopf berlesen:
    do {
        $l = <IN>;
    } while '#' eq substr $l, 0, 1;

    # Eckenzahl einlesen:
    #$l = <IN>;
    die "Fehler in '$dateiname' - N fehlt!" unless defined $l;
    chomp $l;
    (my $n) = $l =~ /^(\d+)$/;
    die "Fehler in '$dateiname' - N fehlt!" unless defined $1;

    # Eckenkoordinaten einlesen:
    for (0..$n-1) {
        $l = <IN>;
        die "Fehler in '$dateiname' - Eckenkoordinate fehlt!" unless defined $l;
        chomp $l;
        $l =~ /^(\d+);(\d+)$/;
        die "Fehler in '$dateiname' - Eckenkoordinate fehlt!"
            unless defined $1 and defined $2;
        $self->ecke_einfuegen($1, $2);
    }


    # Kantenzahl einlesen:
    $l = <IN>;
    die "Fehler in '$dateiname' - M fehlt!" unless defined $l;
    chomp $l;
    (my $m) = $l =~ /^(\d+)$/;
    die "Fehler in '$dateiname' - M fehlt!" unless defined $1;

    # Kantendaten einlesen:
    for (0..$m-1) {
        $l = <IN>;
        die "Fehler in '$dateiname' - Kantendaten fehlen!" unless defined $l;
        chomp $l;
        $l =~ /^(\d+);(\d+);(.*)$/;
        die "Fehler in '$dateiname' - Kantendaten fehlen!"
            unless defined $1 and defined $2 and defined $3;
        $self->kante_einfuegen($1, $2, $3);
    }

    close IN or die "Kann '$dateiname' nicht schlieen: $!";

} # sub Graph::GED::Graph2D::load_ged


sub save_ged2 {
    #--------------------------------------------------------------------------
    # Methode      : s a v e _ g e d 2
    # Klasse       : Graph::GED::Graph2D
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Speichert den Graphen unter dem gegebenen Dateinamen im
    #                ged2-Format.
    # Parameter    : Dateiname
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 01.01.2005 - CD - Erstellt
    # 0.0.2 - 02.01.2005 - CD - Kantendicke und Hintergrundfarbe mit ab-
    #                           speichern.
    # 0.0.3 - 03.01.2005 - CD - Gitterdaten mit speichern.
    #                           Titel- und Gewichtfarbe mit abspeichern.
    # 0.0.4 - 14.01.2005 - CD - Neues Attribut hintergrundbild
    # 0.0.5 - 16.01.2005 - CD - Neues Attribut kantendicke
    # 0.0.6 - 16.01.2005 - CD - Neues Attribut dateiname
    #--------------------------------------------------------------------------

    my $self = shift;

    die "Graph::GED::Graph2D::save_ged2 mit falscher Anzahl Parametern aufgerufen!"
        if scalar @_ != 1;

    my $dateiname = shift;                # Dateiname

    open (OUT, ">$dateiname") or
        die "Kann '$dateiname' nicht zum Schreiben ffnen: $!";

    print OUT <<'HERE';
# Anzahl Ecken
# Anzahl Kanten
# Default-Eckenfarbe
# Default-Kantenfarbe
# Default-Radius
# x-Koordinate <tab> y-Koordinate <tab> Farbe <tab> Radius <tab> Titel <tab> Gewicht <tab> Titelposition <tab> Gewichtposition <tab> Titelfarbe <tab> Gewichtfarbe
# x-Koordinate <tab> y-Koordinate <tab> Farbe <tab> Radius <tab> Titel <tab> Gewicht <tab> Titelposition <tab> Gewichtposition <tab> Titelfarbe <tab> Gewichtfarbe
# x-Koordinate <tab> y-Koordinate <tab> Farbe <tab> Radius <tab> Titel <tab> Gewicht <tab> Titelposition <tab> Gewichtposition <tab> Titelfarbe <tab> Gewichtfarbe
# ...
# x-Koordinate <tab> y-Koordinate <tab> Farbe <tab> Radius <tab> Titel <tab> Gewicht <tab> Titelposition <tab> Gewichtposition <tab> Dicke <tab> Titelfarbe <tab> Gewichtfarbe
# Anfangsecke <tab> Endecke <tab> Form <tab> Farbe <tab> Stil <tab> Titel <tab> Gewicht <tab> Titelposition <tab> Gewichtposition <tab> Dicke <tab> Titelfarbe <tab> Gewichtfarbe
# Anfangsecke <tab> Endecke <tab> Form <tab> Farbe <tab> Stil <tab> Titel <tab> Gewicht <tab> Titelposition <tab> Gewichtposition <tab> Dicke <tab> Titelfarbe <tab> Gewichtfarbe
# ...
# Anfangsecke <tab> Endecke <tab> Form <tab> Farbe <tab> Stil <tab> Titel <tab> Gewicht <tab> Titelposition <tab> Gewichtposition <tab> Dicke <tab> Titelfarbe <tab> Gewichtfarbe
# Default-Hintergrundfarbe
# Gitter zeigen?
# Gitterweite
# Im Gitter Rasten?
# Gitterfarbe
# Hintergrundbild
# Kantendicke
HERE
    print OUT $self->n(), "\n",
              $self->m(), "\n",
              $self->eckenfarbe(), "\n",
              $self->kantenfarbe(), "\n",
              $self->radius(), "\n";
    for my $i (0..$self->n()-1) {
        print OUT $self->ecke($i)->x(),                   "\t",
                  $self->ecke($i)->y(),                   "\t",
                  ($self->ecke($i)->farbe()||'undef'),    "\t",
                  ($self->ecke($i)->radius()||'undef'),   "\t",
                  $self->ecke($i)->titel(),               "\t",
                  ($self->ecke($i)->gewicht()||'undef'),  "\t",
                  $self->ecke($i)->titelpos(),            "\t",
                  $self->ecke($i)->gewichtpos(),          "\t",
                  $self->ecke($i)->titelfarbe(),          "\t",
                  $self->ecke($i)->gewichtfarbe(),        "\n";
    }

    for my $i (0..$self->m()-1) {
        print OUT $self->kante($i)->a(),                  "\t",
                  $self->kante($i)->e(),                  "\t",
                  $self->kante($i)->form(),               "\t",
                  ($self->kante($i)->farbe()||'undef'),   "\t",
                  $self->kante($i)->stil(),               "\t",
                  $self->kante($i)->titel(),              "\t",
                  ($self->kante($i)->gewicht()||'undef'), "\t",
                  $self->kante($i)->titelpos(),           "\t",
                  $self->kante($i)->gewichtpos(),         "\t",
                  ($self->kante($i)->dicke()||'undef'),   "\t",
                  $self->kante($i)->titelfarbe(),         "\t",
                  $self->kante($i)->gewichtfarbe(),       "\n";
    }

    print OUT $self->hintergrund(),      "\n",
              $self->gitterzeigen(),     "\n",
              $self->gitterweite(),      "\n",
              $self->rastimgitter(),     "\n",
              $self->gitterfarbe(),      "\n",
              $self->hintergrundbild(),  "\n",
              $self->kantendicke(),      "\n",
              $self->dateiname(),        "\n";

    close OUT or die "Kann '$dateiname' nicht schlieen: $!";

} # sub Graph::GED::Graph2D::save_ged2


sub load_ged2 {
    #--------------------------------------------------------------------------
    # Methode      : l o a d _ g e d 2
    # Klasse       : Graph::GED::Graph2D
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Ldt den Graphen unter dem gegebenen Dateinamen im
    #                ged2-Format.
    # Parameter    : Dateiname
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 01.01.2005 - CD - Erstellt
    # 0.0.2 - 02.01.2005 - CD - Kantendicke und Hintergrundfarbe mit laden.
    # 0.0.3 - 03.01.2005 - CD - Gitterdaten mit einlesen.
    #                           Titel- und Gewichtfarbe mit einlesen.
    # 0.0.4 - 14.01.2005 - CD - Neues Attribut hintergrundbild
    # 0.0.5 - 16.01.2005 - CD - Neues Attribut kantendicke
    # 0.0.6 - 16.01.2005 - CD - Neues Attribut dateiname
    #--------------------------------------------------------------------------

    my $self = shift;

    die "Graph::GED::Graph2D::load_ged2 mit falscher Anzahl Parametern aufgerufen!"
        if scalar @_ != 1;

    my $dateiname = shift;                # Dateiname

    $self->clear();

    open (IN, "<$dateiname") or
        die "Kann '$dateiname' nicht zum Lesen ffnen: $!";

    my $l = '';

    # Kommentarkopf berlesen:
    do {
        $l = <IN>;
    } while '#' eq substr $l, 0, 1;

    # Eckenzahl einlesen:
    #$l = <IN>;
    die "Fehler in '$dateiname' - N fehlt!" unless defined $l;
    chomp $l;
    (my $n) = $l =~ /^(\d+)$/;
    die "Fehler in '$dateiname' - N fehlt!" unless defined $1;

    # Kantenzahl einlesen:
    $l = <IN>;
    die "Fehler in '$dateiname' - M fehlt!" unless defined $l;
    chomp $l;
    (my $m) = $l =~ /^(\d+)$/;
    die "Fehler in '$dateiname' - M fehlt!" unless defined $1;

    # Eckenfarbe einlesen:
    $l = <IN>;
    die "Fehler in '$dateiname' - Eckenfarbe fehlt!" unless defined $l;
    chomp $l;
    $l =~ /^(.+)$/;
    die "Fehler in '$dateiname' - Eckenfarbe fehlt!" unless defined $1;
    $self->eckenfarbe($1);

    # Kantenfarbe einlesen:
    $l = <IN>;
    die "Fehler in '$dateiname' - Kantenfarbe fehlt!" unless defined $l;
    chomp $l;
    $l =~ /^(.+)$/;
    die "Fehler in '$dateiname' - Kantenfarbe fehlt!" unless defined $1;
    $self->kantenfarbe($1);

    # Radius einlesen:
    $l = <IN>;
    die "Fehler in '$dateiname' - Radius fehlt!" unless defined $l;
    chomp $l;
    $l =~ /^(\d+)$/;
    die "Fehler in '$dateiname' - Radius fehlt!" unless defined $1;
    $self->radius($1);
    $self->r2d();

    # Eckendaten einlesen:
    for (0..$n-1) {
        $l = <IN>;
        die "Fehler in '$dateiname' - Eckendaten fehlen!" unless defined $l;
        chomp $l;
        my ($x, $y, $f, $r, $t, $g, $tp, $gp, $tf, $gf) = split(/\t/, $l);
$tf = '#000000' unless defined $tf; # ---"---
$gf = '#000000' unless defined $gf; # ---"---
        die "Fehler in '$dateiname' - Eckendaten fehlt!"
            unless defined $x
               and defined $y
               and defined $f
               and defined $r
               and defined $t
               and defined $g
               and defined $tp
               and defined $gp
               and defined $tf
               and defined $gf;
        $self->ecke_einfuegen($x, $y);
        $f = undef if $f eq 'undef';
        $r = undef if $r eq 'undef';
        $g = undef if $g eq 'undef';
        $self->ecke($_)->farbe($f);
        $self->ecke($_)->radius($r);
        $self->ecke($_)->titel($t);
        $self->ecke($_)->gewicht($g);
        $self->ecke($_)->titelpos($tp);
        $self->ecke($_)->gewichtpos($gp);
        $self->ecke($_)->titelfarbe($tf);
        $self->ecke($_)->gewichtfarbe($gf);
    }


    # Kantendaten einlesen:
    for (0..$m-1) {
        $l = <IN>;
        die "Fehler in '$dateiname' - Kantendaten fehlen!" unless defined $l;
        chomp $l;

        my ($a, $e, $fo, $fa, $s, $t, $g, $tp, $gp, $d, $tf, $gf) = split(/\t/, $l);
$d = 1 unless defined $d; # bergangszeit zum Einladen lterer ged2-Dateien, die mit Version 0.0.32 erstellt wurden
$tf = '#000000' unless defined $tf; # ---"---
$gf = '#000000' unless defined $gf; # ---"---
        die "Fehler in '$dateiname' - Kantendaten fehlt!"
            unless defined $a
               and defined $e
               and defined $fo
               and defined $fa
               and defined $s
               and defined $t
               and defined $g
               and defined $tp
               and defined $gp
               and defined $d
               and defined $tf
               and defined $gf;
        $self->kante_einfuegen($a, $e, $fo);
        $fa = undef if $fa eq 'undef';
        $g  = undef if $g  eq 'undef';
        $d  = undef if $d  eq 'undef';
        #$self->kante($_)->form($fo); schon durch einfgen gesetzt
        $self->kante($_)->farbe($fa);
        $self->kante($_)->stil($s);
        $self->kante($_)->titel($t);
        $self->kante($_)->gewicht($g);
        $self->kante($_)->titelpos($tp);
        $self->kante($_)->gewichtpos($gp);
        $self->kante($_)->dicke($d);
        $self->kante($_)->titelfarbe($tf);
        $self->kante($_)->gewichtfarbe($gf);
    }

    # Hintergrundfarbe einlesen:
    $l = <IN> if defined $l;
    if (defined $l) { # bergangszeit zum Einladen lterer ged2-Dateien, die mit Version 0.0.32 erstellt wurden
        die "Fehler in '$dateiname' - Hintergrundfarbe fehlt!" unless defined $l;
        chomp $l;
        $l =~ /^(.+)$/;
        die "Fehler in '$dateiname' - Hintergrundfarbe fehlt!" unless defined $1;
        $self->hintergrund($1);
    }

    # Gitterwerte einlesen:
    if (defined $l) {
        $l = <IN>;
        if (defined $l) { # bergangszeit zum Einladen lterer ged2-Dateien, die mit Version 0.0.32/33 erstellt wurden
            die "Fehler in '$dateiname' - gitterzeigen fehlt!" unless defined $l;
            chomp $l;
            $l =~ /^(\d+)$/;
            die "Fehler in '$dateiname' - gitterzeigen fehlt!" unless defined $1;
            $self->gitterzeigen($1);
        }
    }
    if (defined $l) {
        $l = <IN>;
        if (defined $l) { # bergangszeit zum Einladen lterer ged2-Dateien, die mit Version 0.0.32/33 erstellt wurden
            die "Fehler in '$dateiname' - gitterweite fehlt!" unless defined $l;
            chomp $l;
            $l =~ /^(\d+)$/;
            die "Fehler in '$dateiname' - gitterweite fehlt!" unless defined $1;
            $self->gitterweite($1);
        }
    }
    if (defined $l) {
        $l = <IN>;
        if (defined $l) { # bergangszeit zum Einladen lterer ged2-Dateien, die mit Version 0.0.32/33 erstellt wurden
            die "Fehler in '$dateiname' - rastimgitter fehlt!" unless defined $l;
            chomp $l;
            $l =~ /^(\d+)$/;
            die "Fehler in '$dateiname' - rastimgitter fehlt!" unless defined $1;
            $self->rastimgitter($1);
        }
    }
    if (defined $l) {
        $l = <IN>;
        if (defined $l) { # bergangszeit zum Einladen lterer ged2-Dateien, die mit Version 0.0.32/33 erstellt wurden
            die "Fehler in '$dateiname' - gitterfarbe fehlt!" unless defined $l;
            chomp $l;
            $l =~ /^(.+)$/;
            die "Fehler in '$dateiname' - gitterfarbe fehlt!" unless defined $1;
            $self->gitterfarbe($1);
        }
    }

    # Hintergrundbild einlesen:
    $l = <IN> if defined $l;
    if (defined $l) { # bergangszeit zum Einladen lterer ged2-Dateien, die mit Version 0.0.32 erstellt wurden
        die "Fehler in '$dateiname' - Hintergrundbild fehlt!" unless defined $l;
        chomp $l;
        my ($h) = $l =~ /^(.+)$/;
        $h = '' unless defined $h;
        die "Fehler in '$dateiname' - Hintergrundbild fehlt!" unless defined $h;
        $self->hintergrundbild($h);
    }

    # Kantendicke einlesen:
    $l = <IN> if defined $l;
    if (defined $l) { # bergangszeit zum Einladen lterer ged2-Dateien, die mit Version 0.0.32 erstellt wurden
        die "Fehler in '$dateiname' - Kantendicke fehlt!" unless defined $l;
        chomp $l;
        $l =~ /^(\d+)$/;
        die "Fehler in '$dateiname' - Kantendicke fehlt!" unless defined $1;
        $self->kantendicke($1);
    }

    # Dateiname einlesen:
    $l = <IN> if defined $l;
    if (defined $l) { # bergangszeit zum Einladen lterer ged2-Dateien, die mit Version 0.0.32 erstellt wurden
        die "Fehler in '$dateiname' - Dateiname fehlt!" unless defined $l;
        chomp $l;
        my ($d) = $l =~ /^(.+)$/;
        $d = '' unless defined $d;
        die "Fehler in '$dateiname' - Dateiname fehlt!" unless defined $d;
        $self->kantendicke($d);
    }

    close IN or die "Kann '$dateiname' nicht schlieen: $!";

} # sub Graph::GED::Graph2D::load_ged2


sub save_ged3 {
    #--------------------------------------------------------------------------
    # Methode      : s a v e _ g e d 3
    # Klasse       : Graph::GED::Graph2D
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Speichert den Graphen unter dem gegebenen Dateinamen im
    #                ged3-Format.
    # Parameter    : Dateiname
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 16.01.2005 - CD - Erstellt
    #--------------------------------------------------------------------------

    my $self = shift;

    die "Graph::GED::Graph2D::save_ged3 mit falscher Anzahl Parametern aufgerufen!"
        if scalar @_ != 1;

    my $dateiname = shift;                # Dateiname

    open (OUT, ">$dateiname") or
        die "Kann '$dateiname' nicht zum Schreiben oeffnen: $!";

    print OUT "<graph>\n",
              "    <vertexnumber>",       $self->n(),               "</vertexnumber>\n",
              "    <edgenumber>",         $self->m(),               "</edgenumber>\n",
              "    <defaultvertexcolor>", $self->eckenfarbe() ,     "</defaultvertexcolor>\n",
              "    <defaultedgecolor>",   $self->kantenfarbe(),     "</defaultedgecolor>\n",
              "    <defaultradius>",      $self->radius(),          "</defaultradius>\n",
              "    <backgroundcolor>",    $self->hintergrund(),     "</backgroundcolor>\n",
              "    <showgrid>",           $self->gitterzeigen(),    "</showgrid>\n",
              "    <griddistance>",       $self->gitterweite(),     "</griddistance>\n",
              "    <snapingrid>",         $self->rastimgitter(),    "</snapingrid>\n",
              "    <gridcolor>",          $self->gitterfarbe(),     "</gridcolor>\n",
              "    <backgroundpicture>",  $self->hintergrundbild(), "</backgroundpicture>\n",
              "    <defaultedgewidth>",   $self->kantendicke(),     "</defaultedgewidth>\n",
              "    <filename>",           $self->dateiname(),       "</filename>\n",
              "    <vertices>\n";

    for my $i (0..$self->n()-1) {
        print OUT "        <vertex>\n",
                  "            <number>",         $i,                                    "</number>\n",
                  "            <x>",              $self->ecke($i)->x(),                  "</x>\n",
                  "            <y>",              $self->ecke($i)->y(),                  "</y>\n",
                  "            <color>",          ($self->ecke($i)->farbe()||'undef'),   "</color>\n",
                  "            <radius>",         ($self->ecke($i)->radius()||'undef'),  "</radius>\n",
                  "            <title>",          $self->ecke($i)->titel(),              "</title>\n",
                  "            <weight>",         ($self->ecke($i)->gewicht()||'undef'), "</weight>\n",
                  "            <titleposition>",  $self->ecke($i)->titelpos(),           "</titleposition>\n",
                  "            <weightposition>", $self->ecke($i)->gewichtpos(),         "</weightposition>\n",
                  "            <titlecolor>",     $self->ecke($i)->titelfarbe(),         "</titlecolor>\n",
                  "            <weightcolor>",    $self->ecke($i)->gewichtfarbe(),       "</weightcolor>\n",
                  "        </vertex>\n";
    }

    print OUT "    </vertices>\n",
              "    <edges>\n";

    for my $i (0..$self->m()-1) {
        print OUT "        <edge>\n",
                  "            <number>",         $i,                                     "</number>\n",
                  "            <startvertex>",    $self->kante($i)->a(),                  "</startvertex>\n",
                  "            <endvertex>",      $self->kante($i)->e(),                  "</endvertex>\n",
                  "            <form>",           $self->kante($i)->form(),               "</form>\n",
                  "            <color>",          ($self->kante($i)->farbe()||'undef'),   "</color>\n",
                  "            <stil>",           $self->kante($i)->stil(),               "</stil>\n",
                  "            <width>",          ($self->kante($i)->dicke()||'undef'),   "</width>\n",
                  "            <title>",          $self->kante($i)->titel(),              "</title>\n",
                  "            <weight>",         ($self->kante($i)->gewicht()||'undef'), "</weight>\n",
                  "            <titleposition>",  $self->kante($i)->titelpos(),           "</titleposition>\n",
                  "            <weightposition>", $self->kante($i)->gewichtpos(),         "</weightposition>\n",
                  "            <titlecolor>",     $self->kante($i)->titelfarbe(),         "</titlecolor>\n",
                  "            <weightcolor>",    $self->kante($i)->gewichtfarbe(),       "</weightcolor>\n",
                  "        </edge>\n";
    }

    print OUT "    </edges>\n",
              "</graph>\n";

    close OUT or die "Kann '$dateiname' nicht schlieen: $!";

} # sub Graph::GED::Graph2D::save_ged3


sub load_ged3 {
    #--------------------------------------------------------------------------
    # Methode      : l o a d _ g e d 3
    # Klasse       : Graph::GED::Graph2D
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Ldt den Graphen unter dem gegebenen Dateinamen im
    #                ged3-Format.
    # Parameter    : Dateiname
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 16.01.2005 - CD - Erstellt
    #--------------------------------------------------------------------------

    my $self = shift;

    die "Graph::GED::Graph2D::load_ged2 mit falscher Anzahl Parametern aufgerufen!"
        if scalar @_ != 1;

    my $dn = shift; # Dateiname

    $self->clear();

    #--------------------------------------------------------------------------
    # Datei einlesen:
    #--------------------------------------------------------------------------
    open (F, $dn) or die "Kann '$dn' nicht oeffnen: $!";
    my $txt;
    {local $/; $txt = <F>;}
    close F or warn $!;

    #--------------------------------------------------------------------------
    # Inhalt des (ersten) graph-Tags ermitteln:
    #--------------------------------------------------------------------------
    my ($graph) = $txt =~ m~<graph>((?:(?!</graph>).)+)</graph>~s;

    #--------------------------------------------------------------------------
    # Inhalt des vertices- und des edges-Tags ermitteln:
    #--------------------------------------------------------------------------
    $graph =~ s~<vertices>((?:(?!</vertices>).)+)</vertices>~~s;
    my $ecken = $1;
    $graph =~ s~<edges>((?:(?!</edges>).)+)</edges>~~s;
    my $kanten = $1;

    #--------------------------------------------------------------------------
    # Grunddaten ermitteln:
    #--------------------------------------------------------------------------
    while ($graph =~ s~<([a-z]+)>(.*?)</\1>~~) {
        my ($tag, $inhalt) = ($1, $2);

        if    ($tag eq 'vertexnumber')       { $self->n($inhalt) }
        elsif ($tag eq 'edgenumber')         { $self->m($inhalt) }
        elsif ($tag eq 'defaultvertexcolor') { $self->eckenfarbe($inhalt) }
        elsif ($tag eq 'defaultedgecolor')   { $self->kantenfarbe($inhalt) }
        elsif ($tag eq 'defaultradius')      { $self->radius($inhalt) }
        elsif ($tag eq 'backgroundcolor')    { $self->hintergrund($inhalt) }
        elsif ($tag eq 'showgrid')           { $self->gitterzeigen($inhalt) }
        elsif ($tag eq 'griddistance')       { $self->gitterweite($inhalt) }
        elsif ($tag eq 'snapingrid')         { $self->rastimgitter($inhalt) }
        elsif ($tag eq 'gridcolor')          { $self->gitterfarbe($inhalt) }
        elsif ($tag eq 'backgroundpicture')  { $self->hintergrundbild($inhalt) }
        elsif ($tag eq 'defaultedgewidth')   { $self->kantendicke($inhalt) }
        elsif ($tag eq 'filename')           { $self->dateiname($inhalt) }
    }

    #--------------------------------------------------------------------------
    # Eckendaten ermitteln:
    #--------------------------------------------------------------------------
    while ($ecken =~ s~<vertex>((?:(?!</vertex>).)+)</vertex>~~s) {
        my $ecke = $1;
        my $e = new Graph::GED::Ecke;
        while ($ecke =~ s~<([a-z]+)>(.*?)</\1>~~) {
            my ($tag, $inhalt) = ($1, $2);
            if    ($tag eq 'number')         { ;                                                      }
            elsif ($tag eq 'x')              { $e->x           ($inhalt)                              }
            elsif ($tag eq 'y')              { $e->y           ($inhalt)                              }
            elsif ($tag eq 'color')          { $e->farbe       ($inhalt eq 'undef' ? undef : $inhalt) }
            elsif ($tag eq 'radius')         { $e->radius      ($inhalt eq 'undef' ? undef : $inhalt) }
            elsif ($tag eq 'title')          { $e->titel       ($inhalt)                              }
            elsif ($tag eq 'weight')         { $e->gewicht     ($inhalt eq 'undef' ? undef : $inhalt) }
            elsif ($tag eq 'titleposition')  { $e->titelpos    ($inhalt)                              }
            elsif ($tag eq 'weightposition') { $e->gewichtpos  ($inhalt)                              }
            elsif ($tag eq 'titlecolor')     { $e->titelfarbe  ($inhalt)                              }
            elsif ($tag eq 'weightcolor')    { $e->gewichtfarbe($inhalt)                              }
        }
        $self->fuege_ecke_hinzu($e);
    }

    #--------------------------------------------------------------------------
    # Kantendaten ermitteln:
    #--------------------------------------------------------------------------
    while ($kanten =~ s~<edge>((?:(?!</edge>).)+)</edge>~~s) {
        my $kante = $1;
        my $k = new Graph::GED::Kante;
        while ($kante =~ s~<([a-z]+)>(.*?)</\1>~~) {
            my ($tag, $inhalt) = ($1, $2);
            if    ($tag eq 'number')         { ;                                                      }
            elsif ($tag eq 'startvertex')    { $k->a           ($inhalt)                              }
            elsif ($tag eq 'endvertex')      { $k->e           ($inhalt)                              }
            elsif ($tag eq 'form')           { $k->form        ($inhalt)                              }
            elsif ($tag eq 'color')          { $k->farbe       ($inhalt eq 'undef' ? undef : $inhalt) }
            elsif ($tag eq 'stil')           { $k->stil        ($inhalt)                              }
            elsif ($tag eq 'width')          { $k->dicke       ($inhalt eq 'undef' ? undef : $inhalt) }
            elsif ($tag eq 'title')          { $k->titel       ($inhalt)                              }
            elsif ($tag eq 'weight')         { $k->gewicht     ($inhalt eq 'undef' ? undef : $inhalt) }
            elsif ($tag eq 'titleposition')  { $k->titelpos    ($inhalt)                              }
            elsif ($tag eq 'weightposition') { $k->gewichtpos  ($inhalt)                              }
            elsif ($tag eq 'titlecolor')     { $k->titelfarbe  ($inhalt)                              }
            elsif ($tag eq 'weightcolor')    { $k->gewichtfarbe($inhalt)                              }
        }
        $self->fuege_kante_hinzu($k);
    }

} # sub Graph::GED::Graph2D::load_ged3


1;

__END__

=head1 NAME

Graphen::GED::Graph2D.pm - a module for graphs mit two dimensional coordiantes


=head1 METHODS

=head2 class methods

=over

=item C<new>

    Creates an object of this class.

=head2 object methods

=item C<clone>

    Creates a cloned object of this graph.

=item C<radius>

    Getter / Setter for the radius.

=item C<delta>

    Getter / Setter for the delta.

=item C<eckenfarbe>

    Getter / Setter for the standard vertex color.

=item C<kantenfarbe>

    Getter / Setter for the standard edge color.

=item C<hintergrund>

    Getter / Setter for the standard background color.

=item C<gitterzeigen>

    Getter / Setter for the information, if the grid is shown.

=item C<gitterweite>

    Getter / Setter for the grid distance.

=item C<rastimgitter>

    Getter / Setter for the information, if new edges snap into the grid.

=item C<gitterfarbe>

    Getter / Setter for the grid color.

=item C<hintergrundbild>

    Getter / Setter for the background picture.

=item C<kantendicke>

    Getter / Setter for the default width of the edges.

=item C<dateiname>

    Getter / Setter for the file name.

=item C<_ecken>

    Internal getter / setter for the vertex set.

=item C<setze_ecken>

    Sets the vertex set.

=item C<loesche_ecken>

    Deletes the vertex set.

=item C<ecke>

    Getter / Setter (per index) for a vertex of the set.

=item C<entnehme_ecke>

    Deletes a vertex from the graph and returns this vertex.

=item C<fuege_ecke_hinzu>

    Adds a vertex to the graph.

=item C<x_von_a>

    Returns the x-coordinate of the first vertex of the given edge.

=item C<y_von_a>

    Returns the y-coordinate of the first vertex of the given edge.

=item C<x_von_e>

    Returns the x-coordinate of the second vertex of the given edge.

=item C<y_von_e>

    Returns the y-coordinate of the second vertex of the given edge.

=item C<x_von_a_index>

    Returns the x-coordinate of the first vertex of the given edge
    (per index).

=item C<y_von_a_index>

    Returns the y-coordinate of the first vertex of the given edge
    (per index).

=item C<x_von_e_index>

    Returns the x-coordinate of the second vertex of the given edge
    (per index).

=item C<y_von_e_index>

    Returns the y-coordinate of the second vertex of the given edge
    (per index).

=item C<ecke_einfuegen>

    Inserts a vertex.

=item C<ecke_loeschen>

    Deletes a vertex.

=item C<ecke_mergen>

    Merges a vertex (the vertex will be deleted and any neighbours of that
    vertex will be connected through new edges).

=item C<clear>

    Clears everything.

=item C<ecke_zu_nah>

    Tests if a pair of coordinates is too near to any vertex.

=item C<ecke_zu_nah_ohne>

    Like C<ecke_zu_nah> without a given special vertex.

=item C<meint_ecke>

    Tests if a pair of coordinates unequivocal means a vertex of the graph.

=item C<meint_kante>

    Tests if a pair of coordinates unequivocal means an edge of the graph.

=item C<r2d>

    Calculates the internal delta out of the internal radius.

=item C<radius2delta>

    Calculates the delta out of the given radius and returns the delta.

=item C<struktur_einfuegen>

    Inserts a given structure like a circle (K_5).

=item C<erzeuge_liniengraphen>

    Creates the line graph.

=item C<toLaTeX>

    Exports the graph into a file in the LaTeX format.

=item C<nach_links>

    Moves the graph to the left.

=item C<nach_rechts>

    Moves the graph to the right.

=item C<nach_oben>

    Moves the graph to the top.

=item C<nach_unten>

    Moves the graph to the bottom.

=item C<zentrieren>

    Centers the graph.

=item C<zentrieren_skalieren>

    Centers and scales the graph.

=item C<kleiner>

    Makes the graph smaller.

=item C<groesser>

    Makes the graph bigger.

=item C<save_ged>

    Saves the graph into a file in the ged format.

=item C<load_ged>

    Loads a graph from a file in the ged format.

=item C<save_ged2>

    Saves the graph into a file in the ged2 format.

=item C<load_ged2>

    Loads a graph from a file in the ged2 format.

=item C<save_ged3>

    Saves the graph into a file in the ged3 format.

=item C<load_ged3>

    Loads a graph from a file in the ged3 format.

=back

=head1 FORMAT GED

    Anzahl Ecken
    x-Koordinate der ersten Ecke;y-Koordinate der ersten Ecke
    x-Koordinate der zweiten Ecke;y-Koordinate der zweiten Ecke
    ...
    x-Koordinate der letzten Ecke;y-Koordinate der letzten Ecke
    Anzahl Kanten
    Anfangsecke der ersten Kante;Endecke der ersten Kante;Form der ertsen Kante
    Anfangsecke der zweiten Kante;Endecke der zweiten Kante;Form der zweiten Kante
    ...
    Anfangsecke der letzten Kante;Endecke der letzten Kante;Form der letzten Kante

=head1 FORMAT GED2

    Anzahl Ecken
    Anzahl Kanten
    Default-Eckenfarbe
    Default-Kantenfarbe
    Default-Radius
    x-Koordinate <tab> y-Koordinate <tab> Farbe <tab> Radius <tab> Titel <tab> Gewicht <tab> Titelposition <tab> Gewichtposition <tab> Titelfarbe <tab> Gewichtfarbe
    x-Koordinate <tab> y-Koordinate <tab> Farbe <tab> Radius <tab> Titel <tab> Gewicht <tab> Titelposition <tab> Gewichtposition <tab> Titelfarbe <tab> Gewichtfarbe
    x-Koordinate <tab> y-Koordinate <tab> Farbe <tab> Radius <tab> Titel <tab> Gewicht <tab> Titelposition <tab> Gewichtposition <tab> Titelfarbe <tab> Gewichtfarbe
    ...
    x-Koordinate <tab> y-Koordinate <tab> Farbe <tab> Radius <tab> Titel <tab> Gewicht <tab> Titelposition <tab> Gewichtposition <tab> Dicke <tab> Titelfarbe <tab> Gewichtfarbe
    Anfangsecke <tab> Endecke <tab> Form <tab> Farbe <tab> Stil <tab> Titel <tab> Gewicht <tab> Titelposition <tab> Gewichtposition <tab> Dicke <tab> Titelfarbe <tab> Gewichtfarbe
    Anfangsecke <tab> Endecke <tab> Form <tab> Farbe <tab> Stil <tab> Titel <tab> Gewicht <tab> Titelposition <tab> Gewichtposition <tab> Dicke <tab> Titelfarbe <tab> Gewichtfarbe
    ...
    Anfangsecke <tab> Endecke <tab> Form <tab> Farbe <tab> Stil <tab> Titel <tab> Gewicht <tab> Titelposition <tab> Gewichtposition <tab> Dicke <tab> Titelfarbe <tab> Gewichtfarbe
    Default-Hintergrundfarbe
    Gitter zeigen?
    Gitterweite
    Im Gitter Rasten?
    Gitterfarbe
    Hintergrundbild
    Kantendicke

=head1 FORMAT GED3

    <graph>
        <vertexnumber>2</vertexnumber>
        <edgenumber>1</edgenumber>
        <defaultvertexcolor>#FF0000</defaultvertexcolor>
        <defaultedgecolor>#000000</defaultedgecolor>
        <defaultradius>3</defaultradius>
        <backgroundcolor>#FFFFFF</backgroundcolor>
        <showgrid>0</showgrid>
        <griddistance>25</griddistance>
        <snapingrid>0</snapingrid>
        <gridcolor>#C0C0C0</gridcolor>
        <backgroundpic></backgroundpic>
        <defaultedgewidth>1</defaultedgewidth>
        <filename>.../blubb.ged3</filename>
        <vertices>
            <vertex>
                <number>0</number>
                <x>100</x>
                <y>150</y>
                <color>#0000FF</color>
                <radius>10</radius>
                <title>Hamburg</title>
                <weight>undef</weight>
                <titleposition>S</titleposition>
                <weightposition>SO</weightposition>
                <titlecolor>#00FF00</titlecolor>
                <weightcolor>#0000FF</weightcolor>
            </vertex>
            <vertex>
                <number>1</number>
                <x>500</x>
                <y>150</y>
                <color>#0000FF</color>
                <radius>7</radius>
                <title>Bielefeld</title>
                <weight>undef</weight>
                <titleposition>S</titleposition>
                <weightposition>SO</weightposition>
                <titlecolor>#00FF00</titlecolor>
                <weightcolor>#0000FF</weightcolor>
            </vertex>
        </vertices>
        <edges>
            <edge>
                <number>0</number>
                <startvertex>0</startvertex>
                <endvertex>1</endvertex>
                <form>k_ins</form>
                <color>#00AA00</color>
                <stil>normal</stil>
                <width>3</width>
                <title></title>
                <weight>270</weight>
                <titleposition>NW</titleposition>
                <weightposition>S</weightposition>
                <titlecolor>#00FF00</titlecolor>
                <weightcolor>#0000FF</weightcolor>
            </edge>
        </edges>
    </graph>


=head1 COPYRIGHT

Copyright 2004-2005 Christian Duehl. All rights reserved.


This library is free software. You can redistribute it and/or
modify it under the same terms as perl itself.

=cut

