#------------------------------------------------------------------------------
# Programm         : K a n t e . p m
#------------------------------------------------------------------------------
# Autor            : Christian Dhl
# Erstellt         : 10. Februar 2004
# Zuletzt gendert : 16. Januar 2005
# Aufgabe          : Kantenklasse
#------------------------------------------------------------------------------

#------------------------------------------------------------------------------
# Globale private Datei-Variablen:
#------------------------------------------------------------------------------
my $Debug  = 0;


#------------------------------------------------------------------------------
# Klasse           : K a n t e
# abgeleitet von   : none
#------------------------------------------------------------------------------
# Autor            : Christian Dhl
# Beschreibung     : Eine Kanten-Klasse.
# Attribute        : a                  - Anfangsecke
#                    e                  - Endecke
#                    form               - Form der Kante fr die Darstellung
#                    farbe              - Farbe der Kante
#                    stil               - Stil der Kante
#                    titel              - Beschriftung der Kante
#                    gewicht            - Gewicht der Kante
#                    titelpos           - Position der Beschreibung
#                    gewichtpos         - Position des Gewichts
#                    dicke              - Dicke der Kante
#                    titelfarbe         - Position der Beschreibung
#                    gewichtfarbe       - Position des Gewichts
# Klassen-Methoden : new                - Erstellt ein Objekt dieser Klasse.
# Objekt-Methoden  : a                  - Getter / Setter
#                    e                  - Getter / Setter
#                    form               - Getter / Setter
#                    farbe              - Getter / Setter
#                    stil               - Getter / Setter
#                    titel              - Getter / Setter 
#                    gewicht            - Getter / Setter
#                    titelpos           - Getter / Setter
#                    gewichtpos         - Getter / Setter
#                    dicke              - Getter / Setter
#                    titelfarbe         - Getter / Setter
#                    gewichtfarbe       - Getter / Setter
#                    inzidiert_mit_ecke - Test, ob Kante an Ecke stt.
#------------------------------------------------------------------------------
# 0.0.1 - 10.02.2004 - CD - erstellt
# 0.0.2 - 30.12.2004 - CD - Neue Attribute Farbe (farbe), Stil (stil), Be-
#                           schriftung (titel) und Gewicht (gewicht).
# 0.0.3 - 31.12.2004 - CD - Neue Attribute Titelposition (titelpos) und Wert-
#                           position (gewichtpos).
# 0.0.4 - 01.01.2005 - CD - Neues Attribut dicke.
# 0.0.5 - 03.01.2005 - CD - Neue Attribute Titelfarbe (titelfarbe) und
#                           Gewichtfarbe (gewichtfarbe).
#------------------------------------------------------------------------------
package Graph::GED::Kante;


use strict;
use warnings;

use Data::Dumper;


#------------------------------------------------------------------------------
# Globale ffentliche Klassen-Variablen:
#------------------------------------------------------------------------------
use base qw/Graph::GED::Basis/;


sub new {
    #--------------------------------------------------------------------------
    # Methode      : n e w
    # Klasse       : Graph::GED::Kante
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Erzeugt eine neue Kante.
    # Parameter    : Klasse
    # Rckgabewert : Das geblesste neue Objekt.
    #--------------------------------------------------------------------------
    # 0.0.1 - 08.02.2004 - CD - erstellt
    # 0.0.2 - 10.02.2004 - CD - Vereinfachung durch Verwendung der Klasse
    #                           Basis.
    # 0.0.3 - 30.12.2004 - CD - Neue Attribute farbe, stil, titel und gewicht.
    # 0.0.4 - 31.12.2004 - CD - Neue Attribute titelpos und gewichtpos.
    # 0.0.5 - 01.01.2005 - CD - Neues Attribut dicke.
    # 0.0.6 - 03.01.2005 - CD - Neue Attribute titelfarbe und gewichtfarbe.
    # 0.0.7 - 16.01.2005 - CD - Die Kantendicke ist jetzt auch mit Defaultwert
    #                           am Graphen eingetragen. Deshalb wird
    #                           standardmig undef eingetragen.
    #--------------------------------------------------------------------------

    my $self  = shift;
    my $class = (ref $self) || $self;

    #--------------------------------------------------------------------------
    # Ausgabe der Aufrufparameter:
    #--------------------------------------------------------------------------
    print "Graph::GED::Kante::new\n",
          "\tself       = '$self'\n",
          "\tclass      = '$class'\n",
          "\tParameter  = @_\n",
          if $Debug > 1;


    $self = $class->SUPER::new();


    #--------------------------------------------------------------------------
    # Nun setzen wir noch ein paar Attribute:
    #--------------------------------------------------------------------------
    $self->a           (       -1);
    $self->e           (       -1);
    $self->form        (       '');
    $self->farbe       (    undef); # undef : keine individuelle Farbe
    $self->stil        (        0); # 0 normal, 1 gestrichelt, 2 gepunktet
    $self->titel       (       '');
    $self->gewicht     (    undef); # Zahl oder undef fr kein Gewicht
    $self->titelpos    (     'NW'); # Position der Beschreibung
    $self->gewichtpos  (     'SO'); # Position des Gewichts
    $self->dicke       (    undef); # Dicke der Kante in Pixeln
    $self->titelfarbe  ('#000000'); # Farbe der Beschreibung
    $self->gewichtfarbe('#000000'); # Farbe des Gewichts


    #--------------------------------------------------------------------------
    # ... und geben das erzeugte Objekt zurck:
    #--------------------------------------------------------------------------
    return $self;

} # sub Kante::new


#------------------------------------------------------------------------------
# The getter/setter methods:
#------------------------------------------------------------------------------
sub a            { return shift->_property('a',            @_) }
sub e            { return shift->_property('e',            @_) }
sub form         { return shift->_property('form',         @_) }
sub farbe        { return shift->_property('farbe',        @_) }
sub stil         { return shift->_property('stil',         @_) }
sub titel        { return shift->_property('titel',        @_) }
sub gewicht      { return shift->_property('gewicht',      @_) }
sub titelpos     { return shift->_property('titelpos',     @_) }
sub gewichtpos   { return shift->_property('gewichtpos',   @_) }
sub dicke        { return shift->_property('dicke',        @_) }
sub titelfarbe   { return shift->_property('titelfarbe',   @_) }
sub gewichtfarbe { return shift->_property('gewichtfarbe', @_) }


sub inzidiert_mit_ecke {
    #--------------------------------------------------------------------------
    # Methode      : i n z i d i e r t _ m i t _ e c k e
    # Klasse       : Graph::GED::Kante
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Testet, ob die Kante mit der gegebenen Ecke inzidiert.
    # Parameter    : Ecke (Index, nicht Objekt)
    # Rckgabewert : wahr oder falsch
    #--------------------------------------------------------------------------
    # 0.0.1 - 10.02.2004 - CD - erstellt
    #--------------------------------------------------------------------------

    my $self = shift;
    my $ecke = shift;

    return ($self->a() == $ecke or $self->e() == $ecke) ? 1 : 0;
} # sub inzidiert_mit_ecke


1;

__END__

=head1 NAME

Graphen::GED::Kante.pm - a module for a edge of a graph ('Kante' is the german
word for edge).

=head1 METHODS

=head2 class methods

=over

=item C<new>

    creates new object
    
=back

=head2 objekt methods

=over

=item C<a>

    Get/Set the first vertex of that edge.

=item C<e>

    Get/Set the second first vertex of that edge.

=item C<form>

    Get/Set the form of the edge (normal, directed, ...).

=item C<farbe>

    Get/Set the color of the edge.

=item C<stil>

    Get/Set the style of the edge (normal, dotted, ...).

=item C<titel>

    Get/Set the description of the edge.

=item C<gewicht>

    Get/Set the weight of the edge.

=item C<titelpos>

    Get/Set the position of the description of the edge.

=item C<gewichtpos>

    Get/Set the position of the weight of the edge.

=item C<dicke>

    Get/Set the width of the edge.

=item C<titelfarbe>

    Get/Set the color of the description of the edge.


=item C<gewichtfarbe>

    Get/Set the color of the weight of the edge.

=item C<inzidiert_mit_ecke>

    Tests if a given vertex is one of the vertexes of this edge.

=back

=head1 COPYRIGHT

Copyright 2004 Christian Duehl. All rights reserved.

This library is free software. You can redistribute it and/or
modify it under the same terms as perl itself.

=cut
